# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: fx/legacy/gravity/v1/query.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from fxsdk.x.fx.legacy.gravity.v1 import genesis_pb2 as fx_dot_legacy_dot_gravity_dot_v1_dot_genesis__pb2
from fxsdk.x.fx.legacy.gravity.v1 import tx_pb2 as fx_dot_legacy_dot_gravity_dot_v1_dot_tx__pb2
from fxsdk.x.fx.legacy.gravity.v1 import types_pb2 as fx_dot_legacy_dot_gravity_dot_v1_dot_types__pb2
from fxsdk.x.gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2
from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n fx/legacy/gravity/v1/query.proto\x12\rfx.gravity.v1\x1a\"fx/legacy/gravity/v1/genesis.proto\x1a\x1d\x66x/legacy/gravity/v1/tx.proto\x1a fx/legacy/gravity/v1/types.proto\x1a\x14gogoproto/gogo.proto\x1a\x1cgoogle/api/annotations.proto\"\x14\n\x12QueryParamsRequest\"B\n\x13QueryParamsResponse\x12+\n\x06params\x18\x01 \x01(\x0b\x32\x15.fx.gravity.v1.ParamsB\x04\xc8\xde\x1f\x00\"\x1b\n\x19QueryCurrentValsetRequest\"C\n\x1aQueryCurrentValsetResponse\x12%\n\x06valset\x18\x01 \x01(\x0b\x32\x15.fx.gravity.v1.Valset\"*\n\x19QueryValsetRequestRequest\x12\r\n\x05nonce\x18\x01 \x01(\x04\"C\n\x1aQueryValsetRequestResponse\x12%\n\x06valset\x18\x01 \x01(\x0b\x32\x15.fx.gravity.v1.Valset\";\n\x19QueryValsetConfirmRequest\x12\r\n\x05nonce\x18\x01 \x01(\x04\x12\x0f\n\x07\x61\x64\x64ress\x18\x02 \x01(\t\"N\n\x1aQueryValsetConfirmResponse\x12\x30\n\x07\x63onfirm\x18\x01 \x01(\x0b\x32\x1f.fx.gravity.v1.MsgValsetConfirm\"2\n!QueryValsetConfirmsByNonceRequest\x12\r\n\x05nonce\x18\x01 \x01(\x04\"W\n\"QueryValsetConfirmsByNonceResponse\x12\x31\n\x08\x63onfirms\x18\x01 \x03(\x0b\x32\x1f.fx.gravity.v1.MsgValsetConfirm\" \n\x1eQueryLastValsetRequestsRequest\"I\n\x1fQueryLastValsetRequestsResponse\x12&\n\x07valsets\x18\x01 \x03(\x0b\x32\x15.fx.gravity.v1.Valset\"=\n*QueryLastPendingValsetRequestByAddrRequest\x12\x0f\n\x07\x61\x64\x64ress\x18\x01 \x01(\t\"U\n+QueryLastPendingValsetRequestByAddrResponse\x12&\n\x07valsets\x18\x01 \x03(\x0b\x32\x15.fx.gravity.v1.Valset\"N\n\x14QueryBatchFeeRequest\x12\x36\n\x0cminBatchFees\x18\x01 \x03(\x0b\x32\x1a.fx.gravity.v1.MinBatchFeeB\x04\xc8\xde\x1f\x00\"E\n\x15QueryBatchFeeResponse\x12,\n\nbatch_fees\x18\x01 \x03(\x0b\x32\x18.fx.gravity.v1.BatchFees\"<\n)QueryLastPendingBatchRequestByAddrRequest\x12\x0f\n\x07\x61\x64\x64ress\x18\x01 \x01(\t\"[\n*QueryLastPendingBatchRequestByAddrResponse\x12-\n\x05\x62\x61tch\x18\x01 \x01(\x0b\x32\x1e.fx.gravity.v1.OutgoingTxBatch\"\x1f\n\x1dQueryOutgoingTxBatchesRequest\"Q\n\x1eQueryOutgoingTxBatchesResponse\x12/\n\x07\x62\x61tches\x18\x01 \x03(\x0b\x32\x1e.fx.gravity.v1.OutgoingTxBatch\"H\n\x1fQueryBatchRequestByNonceRequest\x12\r\n\x05nonce\x18\x01 \x01(\x04\x12\x16\n\x0etoken_contract\x18\x02 \x01(\t\"Q\n QueryBatchRequestByNonceResponse\x12-\n\x05\x62\x61tch\x18\x01 \x01(\x0b\x32\x1e.fx.gravity.v1.OutgoingTxBatch\"R\n\x18QueryBatchConfirmRequest\x12\r\n\x05nonce\x18\x01 \x01(\x04\x12\x16\n\x0etoken_contract\x18\x02 \x01(\t\x12\x0f\n\x07\x61\x64\x64ress\x18\x03 \x01(\t\"L\n\x19QueryBatchConfirmResponse\x12/\n\x07\x63onfirm\x18\x01 \x01(\x0b\x32\x1e.fx.gravity.v1.MsgConfirmBatch\"B\n\x19QueryBatchConfirmsRequest\x12\r\n\x05nonce\x18\x01 \x01(\x04\x12\x16\n\x0etoken_contract\x18\x02 \x01(\t\"N\n\x1aQueryBatchConfirmsResponse\x12\x30\n\x08\x63onfirms\x18\x01 \x03(\x0b\x32\x1e.fx.gravity.v1.MsgConfirmBatch\"3\n QueryLastEventNonceByAddrRequest\x12\x0f\n\x07\x61\x64\x64ress\x18\x01 \x01(\t\"8\n!QueryLastEventNonceByAddrResponse\x12\x13\n\x0b\x65vent_nonce\x18\x01 \x01(\x04\")\n\x18QueryERC20ToDenomRequest\x12\r\n\x05\x65rc20\x18\x01 \x01(\t\"A\n\x19QueryERC20ToDenomResponse\x12\r\n\x05\x64\x65nom\x18\x01 \x01(\t\x12\x15\n\rfx_originated\x18\x02 \x01(\x08\")\n\x18QueryDenomToERC20Request\x12\r\n\x05\x64\x65nom\x18\x01 \x01(\t\"A\n\x19QueryDenomToERC20Response\x12\r\n\x05\x65rc20\x18\x01 \x01(\t\x12\x15\n\rfx_originated\x18\x02 \x01(\x08\"?\n\"QueryDelegateKeyByValidatorRequest\x12\x19\n\x11validator_address\x18\x01 \x01(\t\"X\n#QueryDelegateKeyByValidatorResponse\x12\x13\n\x0b\x65th_address\x18\x01 \x01(\t\x12\x1c\n\x14orchestrator_address\x18\x02 \x01(\t\"3\n\x1cQueryDelegateKeyByEthRequest\x12\x13\n\x0b\x65th_address\x18\x01 \x01(\t\"X\n\x1dQueryDelegateKeyByEthResponse\x12\x19\n\x11validator_address\x18\x01 \x01(\t\x12\x1c\n\x14orchestrator_address\x18\x02 \x01(\t\"E\n%QueryDelegateKeyByOrchestratorRequest\x12\x1c\n\x14orchestrator_address\x18\x01 \x01(\t\"X\n&QueryDelegateKeyByOrchestratorResponse\x12\x19\n\x11validator_address\x18\x01 \x01(\t\x12\x13\n\x0b\x65th_address\x18\x02 \x01(\t\"6\n\x1cQueryPendingSendToEthRequest\x12\x16\n\x0esender_address\x18\x01 \x01(\t\"\xa0\x01\n\x1dQueryPendingSendToEthResponse\x12?\n\x14transfers_in_batches\x18\x01 \x03(\x0b\x32!.fx.gravity.v1.OutgoingTransferTx\x12>\n\x13unbatched_transfers\x18\x02 \x03(\x0b\x32!.fx.gravity.v1.OutgoingTransferTx\"%\n#QueryLastObservedBlockHeightRequest\"V\n$QueryLastObservedBlockHeightResponse\x12\x18\n\x10\x65th_block_height\x18\x01 \x01(\x04\x12\x14\n\x0c\x62lock_height\x18\x02 \x01(\x04\"9\n&QueryLastEventBlockHeightByAddrRequest\x12\x0f\n\x07\x61\x64\x64ress\x18\x01 \x01(\t\"?\n\'QueryLastEventBlockHeightByAddrResponse\x12\x14\n\x0c\x62lock_height\x18\x01 \x01(\x04\")\n\'QueryProjectedBatchTimeoutHeightRequest\"B\n(QueryProjectedBatchTimeoutHeightResponse\x12\x16\n\x0etimeout_height\x18\x01 \x01(\x04\"\x1a\n\x18QueryBridgeTokensRequest\"O\n\x19QueryBridgeTokensResponse\x12\x32\n\rbridge_tokens\x18\x01 \x03(\x0b\x32\x1b.fx.gravity.v1.ERC20ToDenom2\xe0\x1d\n\x05Query\x12n\n\x06Params\x12!.fx.gravity.v1.QueryParamsRequest\x1a\".fx.gravity.v1.QueryParamsResponse\"\x1d\x82\xd3\xe4\x93\x02\x17\x12\x15/fx/gravity/v1/params\x12\x8b\x01\n\rCurrentValset\x12(.fx.gravity.v1.QueryCurrentValsetRequest\x1a).fx.gravity.v1.QueryCurrentValsetResponse\"%\x82\xd3\xe4\x93\x02\x1f\x12\x1d/fx/gravity/v1/valset/current\x12\x8b\x01\n\rValsetRequest\x12(.fx.gravity.v1.QueryValsetRequestRequest\x1a).fx.gravity.v1.QueryValsetRequestResponse\"%\x82\xd3\xe4\x93\x02\x1f\x12\x1d/fx/gravity/v1/valset/request\x12\x8b\x01\n\rValsetConfirm\x12(.fx.gravity.v1.QueryValsetConfirmRequest\x1a).fx.gravity.v1.QueryValsetConfirmResponse\"%\x82\xd3\xe4\x93\x02\x1f\x12\x1d/fx/gravity/v1/valset/confirm\x12\xa4\x01\n\x15ValsetConfirmsByNonce\x12\x30.fx.gravity.v1.QueryValsetConfirmsByNonceRequest\x1a\x31.fx.gravity.v1.QueryValsetConfirmsByNonceResponse\"&\x82\xd3\xe4\x93\x02 \x12\x1e/fx/gravity/v1/valset/confirms\x12\x9b\x01\n\x12LastValsetRequests\x12-.fx.gravity.v1.QueryLastValsetRequestsRequest\x1a..fx.gravity.v1.QueryLastValsetRequestsResponse\"&\x82\xd3\xe4\x93\x02 \x12\x1e/fx/gravity/v1/valset/requests\x12\xbb\x01\n\x1eLastPendingValsetRequestByAddr\x12\x39.fx.gravity.v1.QueryLastPendingValsetRequestByAddrRequest\x1a:.fx.gravity.v1.QueryLastPendingValsetRequestByAddrResponse\"\"\x82\xd3\xe4\x93\x02\x1c\x12\x1a/fx/gravity/v1/valset/last\x12\xb7\x01\n\x1dLastPendingBatchRequestByAddr\x12\x38.fx.gravity.v1.QueryLastPendingBatchRequestByAddrRequest\x1a\x39.fx.gravity.v1.QueryLastPendingBatchRequestByAddrResponse\"!\x82\xd3\xe4\x93\x02\x1b\x12\x19/fx/gravity/v1/batch/last\x12\xae\x01\n\x14LastEventNonceByAddr\x12/.fx.gravity.v1.QueryLastEventNonceByAddrRequest\x1a\x30.fx.gravity.v1.QueryLastEventNonceByAddrResponse\"3\x82\xd3\xe4\x93\x02-\x12+/fx/gravity/v1/oracle/event_nonce/{address}\x12\xc7\x01\n\x1aLastEventBlockHeightByAddr\x12\x35.fx.gravity.v1.QueryLastEventBlockHeightByAddrRequest\x1a\x36.fx.gravity.v1.QueryLastEventBlockHeightByAddrResponse\":\x82\xd3\xe4\x93\x02\x34\x12\x32/fx/gravity/v1/oracle/event/block_height/{address}\x12y\n\tBatchFees\x12#.fx.gravity.v1.QueryBatchFeeRequest\x1a$.fx.gravity.v1.QueryBatchFeeResponse\"!\x82\xd3\xe4\x93\x02\x1b\x12\x19/fx/gravity/v1/batch_fees\x12\xb0\x01\n\x17LastObservedBlockHeight\x12\x32.fx.gravity.v1.QueryLastObservedBlockHeightRequest\x1a\x33.fx.gravity.v1.QueryLastObservedBlockHeightResponse\",\x82\xd3\xe4\x93\x02&\x12$/fx/gravity/v1/observed/block_height\x12\x9a\x01\n\x11OutgoingTxBatches\x12,.fx.gravity.v1.QueryOutgoingTxBatchesRequest\x1a-.fx.gravity.v1.QueryOutgoingTxBatchesResponse\"(\x82\xd3\xe4\x93\x02\"\x12 /fx/gravity/v1/batch/outgoing_tx\x12\x9c\x01\n\x13\x42\x61tchRequestByNonce\x12..fx.gravity.v1.QueryBatchRequestByNonceRequest\x1a/.fx.gravity.v1.QueryBatchRequestByNonceResponse\"$\x82\xd3\xe4\x93\x02\x1e\x12\x1c/fx/gravity/v1/batch/request\x12\x87\x01\n\x0c\x42\x61tchConfirm\x12\'.fx.gravity.v1.QueryBatchConfirmRequest\x1a(.fx.gravity.v1.QueryBatchConfirmResponse\"$\x82\xd3\xe4\x93\x02\x1e\x12\x1c/fx/gravity/v1/batch/confirm\x12\x8b\x01\n\rBatchConfirms\x12(.fx.gravity.v1.QueryBatchConfirmsRequest\x1a).fx.gravity.v1.QueryBatchConfirmsResponse\"%\x82\xd3\xe4\x93\x02\x1f\x12\x1d/fx/gravity/v1/batch/confirms\x12\x7f\n\x0c\x45RC20ToDenom\x12\'.fx.gravity.v1.QueryERC20ToDenomRequest\x1a(.fx.gravity.v1.QueryERC20ToDenomResponse\"\x1c\x82\xd3\xe4\x93\x02\x16\x12\x14/fx/gravity/v1/denom\x12\x7f\n\x0c\x44\x65nomToERC20\x12\'.fx.gravity.v1.QueryDenomToERC20Request\x1a(.fx.gravity.v1.QueryDenomToERC20Response\"\x1c\x82\xd3\xe4\x93\x02\x16\x12\x14/fx/gravity/v1/erc20\x12\xb4\x01\n\x19GetDelegateKeyByValidator\x12\x31.fx.gravity.v1.QueryDelegateKeyByValidatorRequest\x1a\x32.fx.gravity.v1.QueryDelegateKeyByValidatorResponse\"0\x82\xd3\xe4\x93\x02*\x12(/fx/gravity/v1/delegate_key_by_validator\x12\x9c\x01\n\x13GetDelegateKeyByEth\x12+.fx.gravity.v1.QueryDelegateKeyByEthRequest\x1a,.fx.gravity.v1.QueryDelegateKeyByEthResponse\"*\x82\xd3\xe4\x93\x02$\x12\"/fx/gravity/v1/delegate_key_by_eth\x12\xc0\x01\n\x1cGetDelegateKeyByOrchestrator\x12\x34.fx.gravity.v1.QueryDelegateKeyByOrchestratorRequest\x1a\x35.fx.gravity.v1.QueryDelegateKeyByOrchestratorResponse\"3\x82\xd3\xe4\x93\x02-\x12+/fx/gravity/v1/delegate_key_by_orchestrator\x12\x9c\x01\n\x13GetPendingSendToEth\x12+.fx.gravity.v1.QueryPendingSendToEthRequest\x1a,.fx.gravity.v1.QueryPendingSendToEthResponse\"*\x82\xd3\xe4\x93\x02$\x12\"/fx/gravity/v1/pending_send_to_eth\x12\xbe\x01\n\x1bProjectedBatchTimeoutHeight\x12\x36.fx.gravity.v1.QueryProjectedBatchTimeoutHeightRequest\x1a\x37.fx.gravity.v1.QueryProjectedBatchTimeoutHeightResponse\".\x82\xd3\xe4\x93\x02(\x12&/fx/gravity/v1/projected_batch_timeout\x12\x87\x01\n\x0c\x42ridgeTokens\x12\'.fx.gravity.v1.QueryBridgeTokensRequest\x1a(.fx.gravity.v1.QueryBridgeTokensResponse\"$\x82\xd3\xe4\x93\x02\x1e\x12\x1c/fx/gravity/v1/bridge_tokensB.Z,github.com/functionx/fx-core/x/gravity/typesb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'fx.legacy.gravity.v1.query_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z,github.com/functionx/fx-core/x/gravity/types'
  _QUERYPARAMSRESPONSE.fields_by_name['params']._options = None
  _QUERYPARAMSRESPONSE.fields_by_name['params']._serialized_options = b'\310\336\037\000'
  _QUERYBATCHFEEREQUEST.fields_by_name['minBatchFees']._options = None
  _QUERYBATCHFEEREQUEST.fields_by_name['minBatchFees']._serialized_options = b'\310\336\037\000'
  _QUERY.methods_by_name['Params']._options = None
  _QUERY.methods_by_name['Params']._serialized_options = b'\202\323\344\223\002\027\022\025/fx/gravity/v1/params'
  _QUERY.methods_by_name['CurrentValset']._options = None
  _QUERY.methods_by_name['CurrentValset']._serialized_options = b'\202\323\344\223\002\037\022\035/fx/gravity/v1/valset/current'
  _QUERY.methods_by_name['ValsetRequest']._options = None
  _QUERY.methods_by_name['ValsetRequest']._serialized_options = b'\202\323\344\223\002\037\022\035/fx/gravity/v1/valset/request'
  _QUERY.methods_by_name['ValsetConfirm']._options = None
  _QUERY.methods_by_name['ValsetConfirm']._serialized_options = b'\202\323\344\223\002\037\022\035/fx/gravity/v1/valset/confirm'
  _QUERY.methods_by_name['ValsetConfirmsByNonce']._options = None
  _QUERY.methods_by_name['ValsetConfirmsByNonce']._serialized_options = b'\202\323\344\223\002 \022\036/fx/gravity/v1/valset/confirms'
  _QUERY.methods_by_name['LastValsetRequests']._options = None
  _QUERY.methods_by_name['LastValsetRequests']._serialized_options = b'\202\323\344\223\002 \022\036/fx/gravity/v1/valset/requests'
  _QUERY.methods_by_name['LastPendingValsetRequestByAddr']._options = None
  _QUERY.methods_by_name['LastPendingValsetRequestByAddr']._serialized_options = b'\202\323\344\223\002\034\022\032/fx/gravity/v1/valset/last'
  _QUERY.methods_by_name['LastPendingBatchRequestByAddr']._options = None
  _QUERY.methods_by_name['LastPendingBatchRequestByAddr']._serialized_options = b'\202\323\344\223\002\033\022\031/fx/gravity/v1/batch/last'
  _QUERY.methods_by_name['LastEventNonceByAddr']._options = None
  _QUERY.methods_by_name['LastEventNonceByAddr']._serialized_options = b'\202\323\344\223\002-\022+/fx/gravity/v1/oracle/event_nonce/{address}'
  _QUERY.methods_by_name['LastEventBlockHeightByAddr']._options = None
  _QUERY.methods_by_name['LastEventBlockHeightByAddr']._serialized_options = b'\202\323\344\223\0024\0222/fx/gravity/v1/oracle/event/block_height/{address}'
  _QUERY.methods_by_name['BatchFees']._options = None
  _QUERY.methods_by_name['BatchFees']._serialized_options = b'\202\323\344\223\002\033\022\031/fx/gravity/v1/batch_fees'
  _QUERY.methods_by_name['LastObservedBlockHeight']._options = None
  _QUERY.methods_by_name['LastObservedBlockHeight']._serialized_options = b'\202\323\344\223\002&\022$/fx/gravity/v1/observed/block_height'
  _QUERY.methods_by_name['OutgoingTxBatches']._options = None
  _QUERY.methods_by_name['OutgoingTxBatches']._serialized_options = b'\202\323\344\223\002\"\022 /fx/gravity/v1/batch/outgoing_tx'
  _QUERY.methods_by_name['BatchRequestByNonce']._options = None
  _QUERY.methods_by_name['BatchRequestByNonce']._serialized_options = b'\202\323\344\223\002\036\022\034/fx/gravity/v1/batch/request'
  _QUERY.methods_by_name['BatchConfirm']._options = None
  _QUERY.methods_by_name['BatchConfirm']._serialized_options = b'\202\323\344\223\002\036\022\034/fx/gravity/v1/batch/confirm'
  _QUERY.methods_by_name['BatchConfirms']._options = None
  _QUERY.methods_by_name['BatchConfirms']._serialized_options = b'\202\323\344\223\002\037\022\035/fx/gravity/v1/batch/confirms'
  _QUERY.methods_by_name['ERC20ToDenom']._options = None
  _QUERY.methods_by_name['ERC20ToDenom']._serialized_options = b'\202\323\344\223\002\026\022\024/fx/gravity/v1/denom'
  _QUERY.methods_by_name['DenomToERC20']._options = None
  _QUERY.methods_by_name['DenomToERC20']._serialized_options = b'\202\323\344\223\002\026\022\024/fx/gravity/v1/erc20'
  _QUERY.methods_by_name['GetDelegateKeyByValidator']._options = None
  _QUERY.methods_by_name['GetDelegateKeyByValidator']._serialized_options = b'\202\323\344\223\002*\022(/fx/gravity/v1/delegate_key_by_validator'
  _QUERY.methods_by_name['GetDelegateKeyByEth']._options = None
  _QUERY.methods_by_name['GetDelegateKeyByEth']._serialized_options = b'\202\323\344\223\002$\022\"/fx/gravity/v1/delegate_key_by_eth'
  _QUERY.methods_by_name['GetDelegateKeyByOrchestrator']._options = None
  _QUERY.methods_by_name['GetDelegateKeyByOrchestrator']._serialized_options = b'\202\323\344\223\002-\022+/fx/gravity/v1/delegate_key_by_orchestrator'
  _QUERY.methods_by_name['GetPendingSendToEth']._options = None
  _QUERY.methods_by_name['GetPendingSendToEth']._serialized_options = b'\202\323\344\223\002$\022\"/fx/gravity/v1/pending_send_to_eth'
  _QUERY.methods_by_name['ProjectedBatchTimeoutHeight']._options = None
  _QUERY.methods_by_name['ProjectedBatchTimeoutHeight']._serialized_options = b'\202\323\344\223\002(\022&/fx/gravity/v1/projected_batch_timeout'
  _QUERY.methods_by_name['BridgeTokens']._options = None
  _QUERY.methods_by_name['BridgeTokens']._serialized_options = b'\202\323\344\223\002\036\022\034/fx/gravity/v1/bridge_tokens'
  _globals['_QUERYPARAMSREQUEST']._serialized_start=204
  _globals['_QUERYPARAMSREQUEST']._serialized_end=224
  _globals['_QUERYPARAMSRESPONSE']._serialized_start=226
  _globals['_QUERYPARAMSRESPONSE']._serialized_end=292
  _globals['_QUERYCURRENTVALSETREQUEST']._serialized_start=294
  _globals['_QUERYCURRENTVALSETREQUEST']._serialized_end=321
  _globals['_QUERYCURRENTVALSETRESPONSE']._serialized_start=323
  _globals['_QUERYCURRENTVALSETRESPONSE']._serialized_end=390
  _globals['_QUERYVALSETREQUESTREQUEST']._serialized_start=392
  _globals['_QUERYVALSETREQUESTREQUEST']._serialized_end=434
  _globals['_QUERYVALSETREQUESTRESPONSE']._serialized_start=436
  _globals['_QUERYVALSETREQUESTRESPONSE']._serialized_end=503
  _globals['_QUERYVALSETCONFIRMREQUEST']._serialized_start=505
  _globals['_QUERYVALSETCONFIRMREQUEST']._serialized_end=564
  _globals['_QUERYVALSETCONFIRMRESPONSE']._serialized_start=566
  _globals['_QUERYVALSETCONFIRMRESPONSE']._serialized_end=644
  _globals['_QUERYVALSETCONFIRMSBYNONCEREQUEST']._serialized_start=646
  _globals['_QUERYVALSETCONFIRMSBYNONCEREQUEST']._serialized_end=696
  _globals['_QUERYVALSETCONFIRMSBYNONCERESPONSE']._serialized_start=698
  _globals['_QUERYVALSETCONFIRMSBYNONCERESPONSE']._serialized_end=785
  _globals['_QUERYLASTVALSETREQUESTSREQUEST']._serialized_start=787
  _globals['_QUERYLASTVALSETREQUESTSREQUEST']._serialized_end=819
  _globals['_QUERYLASTVALSETREQUESTSRESPONSE']._serialized_start=821
  _globals['_QUERYLASTVALSETREQUESTSRESPONSE']._serialized_end=894
  _globals['_QUERYLASTPENDINGVALSETREQUESTBYADDRREQUEST']._serialized_start=896
  _globals['_QUERYLASTPENDINGVALSETREQUESTBYADDRREQUEST']._serialized_end=957
  _globals['_QUERYLASTPENDINGVALSETREQUESTBYADDRRESPONSE']._serialized_start=959
  _globals['_QUERYLASTPENDINGVALSETREQUESTBYADDRRESPONSE']._serialized_end=1044
  _globals['_QUERYBATCHFEEREQUEST']._serialized_start=1046
  _globals['_QUERYBATCHFEEREQUEST']._serialized_end=1124
  _globals['_QUERYBATCHFEERESPONSE']._serialized_start=1126
  _globals['_QUERYBATCHFEERESPONSE']._serialized_end=1195
  _globals['_QUERYLASTPENDINGBATCHREQUESTBYADDRREQUEST']._serialized_start=1197
  _globals['_QUERYLASTPENDINGBATCHREQUESTBYADDRREQUEST']._serialized_end=1257
  _globals['_QUERYLASTPENDINGBATCHREQUESTBYADDRRESPONSE']._serialized_start=1259
  _globals['_QUERYLASTPENDINGBATCHREQUESTBYADDRRESPONSE']._serialized_end=1350
  _globals['_QUERYOUTGOINGTXBATCHESREQUEST']._serialized_start=1352
  _globals['_QUERYOUTGOINGTXBATCHESREQUEST']._serialized_end=1383
  _globals['_QUERYOUTGOINGTXBATCHESRESPONSE']._serialized_start=1385
  _globals['_QUERYOUTGOINGTXBATCHESRESPONSE']._serialized_end=1466
  _globals['_QUERYBATCHREQUESTBYNONCEREQUEST']._serialized_start=1468
  _globals['_QUERYBATCHREQUESTBYNONCEREQUEST']._serialized_end=1540
  _globals['_QUERYBATCHREQUESTBYNONCERESPONSE']._serialized_start=1542
  _globals['_QUERYBATCHREQUESTBYNONCERESPONSE']._serialized_end=1623
  _globals['_QUERYBATCHCONFIRMREQUEST']._serialized_start=1625
  _globals['_QUERYBATCHCONFIRMREQUEST']._serialized_end=1707
  _globals['_QUERYBATCHCONFIRMRESPONSE']._serialized_start=1709
  _globals['_QUERYBATCHCONFIRMRESPONSE']._serialized_end=1785
  _globals['_QUERYBATCHCONFIRMSREQUEST']._serialized_start=1787
  _globals['_QUERYBATCHCONFIRMSREQUEST']._serialized_end=1853
  _globals['_QUERYBATCHCONFIRMSRESPONSE']._serialized_start=1855
  _globals['_QUERYBATCHCONFIRMSRESPONSE']._serialized_end=1933
  _globals['_QUERYLASTEVENTNONCEBYADDRREQUEST']._serialized_start=1935
  _globals['_QUERYLASTEVENTNONCEBYADDRREQUEST']._serialized_end=1986
  _globals['_QUERYLASTEVENTNONCEBYADDRRESPONSE']._serialized_start=1988
  _globals['_QUERYLASTEVENTNONCEBYADDRRESPONSE']._serialized_end=2044
  _globals['_QUERYERC20TODENOMREQUEST']._serialized_start=2046
  _globals['_QUERYERC20TODENOMREQUEST']._serialized_end=2087
  _globals['_QUERYERC20TODENOMRESPONSE']._serialized_start=2089
  _globals['_QUERYERC20TODENOMRESPONSE']._serialized_end=2154
  _globals['_QUERYDENOMTOERC20REQUEST']._serialized_start=2156
  _globals['_QUERYDENOMTOERC20REQUEST']._serialized_end=2197
  _globals['_QUERYDENOMTOERC20RESPONSE']._serialized_start=2199
  _globals['_QUERYDENOMTOERC20RESPONSE']._serialized_end=2264
  _globals['_QUERYDELEGATEKEYBYVALIDATORREQUEST']._serialized_start=2266
  _globals['_QUERYDELEGATEKEYBYVALIDATORREQUEST']._serialized_end=2329
  _globals['_QUERYDELEGATEKEYBYVALIDATORRESPONSE']._serialized_start=2331
  _globals['_QUERYDELEGATEKEYBYVALIDATORRESPONSE']._serialized_end=2419
  _globals['_QUERYDELEGATEKEYBYETHREQUEST']._serialized_start=2421
  _globals['_QUERYDELEGATEKEYBYETHREQUEST']._serialized_end=2472
  _globals['_QUERYDELEGATEKEYBYETHRESPONSE']._serialized_start=2474
  _globals['_QUERYDELEGATEKEYBYETHRESPONSE']._serialized_end=2562
  _globals['_QUERYDELEGATEKEYBYORCHESTRATORREQUEST']._serialized_start=2564
  _globals['_QUERYDELEGATEKEYBYORCHESTRATORREQUEST']._serialized_end=2633
  _globals['_QUERYDELEGATEKEYBYORCHESTRATORRESPONSE']._serialized_start=2635
  _globals['_QUERYDELEGATEKEYBYORCHESTRATORRESPONSE']._serialized_end=2723
  _globals['_QUERYPENDINGSENDTOETHREQUEST']._serialized_start=2725
  _globals['_QUERYPENDINGSENDTOETHREQUEST']._serialized_end=2779
  _globals['_QUERYPENDINGSENDTOETHRESPONSE']._serialized_start=2782
  _globals['_QUERYPENDINGSENDTOETHRESPONSE']._serialized_end=2942
  _globals['_QUERYLASTOBSERVEDBLOCKHEIGHTREQUEST']._serialized_start=2944
  _globals['_QUERYLASTOBSERVEDBLOCKHEIGHTREQUEST']._serialized_end=2981
  _globals['_QUERYLASTOBSERVEDBLOCKHEIGHTRESPONSE']._serialized_start=2983
  _globals['_QUERYLASTOBSERVEDBLOCKHEIGHTRESPONSE']._serialized_end=3069
  _globals['_QUERYLASTEVENTBLOCKHEIGHTBYADDRREQUEST']._serialized_start=3071
  _globals['_QUERYLASTEVENTBLOCKHEIGHTBYADDRREQUEST']._serialized_end=3128
  _globals['_QUERYLASTEVENTBLOCKHEIGHTBYADDRRESPONSE']._serialized_start=3130
  _globals['_QUERYLASTEVENTBLOCKHEIGHTBYADDRRESPONSE']._serialized_end=3193
  _globals['_QUERYPROJECTEDBATCHTIMEOUTHEIGHTREQUEST']._serialized_start=3195
  _globals['_QUERYPROJECTEDBATCHTIMEOUTHEIGHTREQUEST']._serialized_end=3236
  _globals['_QUERYPROJECTEDBATCHTIMEOUTHEIGHTRESPONSE']._serialized_start=3238
  _globals['_QUERYPROJECTEDBATCHTIMEOUTHEIGHTRESPONSE']._serialized_end=3304
  _globals['_QUERYBRIDGETOKENSREQUEST']._serialized_start=3306
  _globals['_QUERYBRIDGETOKENSREQUEST']._serialized_end=3332
  _globals['_QUERYBRIDGETOKENSRESPONSE']._serialized_start=3334
  _globals['_QUERYBRIDGETOKENSRESPONSE']._serialized_end=3413
  _globals['_QUERY']._serialized_start=3416
  _globals['_QUERY']._serialized_end=7224
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: fx/erc20/v1/query.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from fxsdk.x.cosmos.base.query.v1beta1 import pagination_pb2 as cosmos_dot_base_dot_query_dot_v1beta1_dot_pagination__pb2
from fxsdk.x.fx.erc20.v1 import erc20_pb2 as fx_dot_erc20_dot_v1_dot_erc20__pb2
from fxsdk.x.fx.erc20.v1 import genesis_pb2 as fx_dot_erc20_dot_v1_dot_genesis__pb2
from fxsdk.x.gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2
from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x17\x66x/erc20/v1/query.proto\x12\x0b\x66x.erc20.v1\x1a*cosmos/base/query/v1beta1/pagination.proto\x1a\x17\x66x/erc20/v1/erc20.proto\x1a\x19\x66x/erc20/v1/genesis.proto\x1a\x14gogoproto/gogo.proto\x1a\x1cgoogle/api/annotations.proto\"T\n\x16QueryTokenPairsRequest\x12:\n\npagination\x18\x01 \x01(\x0b\x32&.cosmos.base.query.v1beta1.PageRequest\"\x89\x01\n\x17QueryTokenPairsResponse\x12\x31\n\x0btoken_pairs\x18\x01 \x03(\x0b\x32\x16.fx.erc20.v1.TokenPairB\x04\xc8\xde\x1f\x00\x12;\n\npagination\x18\x02 \x01(\x0b\x32\'.cosmos.base.query.v1beta1.PageResponse\"&\n\x15QueryTokenPairRequest\x12\r\n\x05token\x18\x01 \x01(\t\"J\n\x16QueryTokenPairResponse\x12\x30\n\ntoken_pair\x18\x01 \x01(\x0b\x32\x16.fx.erc20.v1.TokenPairB\x04\xc8\xde\x1f\x00\"\x14\n\x12QueryParamsRequest\"@\n\x13QueryParamsResponse\x12)\n\x06params\x18\x01 \x01(\x0b\x32\x13.fx.erc20.v1.ParamsB\x04\xc8\xde\x1f\x00\")\n\x18QueryDenomAliasesRequest\x12\r\n\x05\x64\x65nom\x18\x01 \x01(\t\",\n\x19QueryDenomAliasesResponse\x12\x0f\n\x07\x61liases\x18\x01 \x03(\t\"\'\n\x16QueryAliasDenomRequest\x12\r\n\x05\x61lias\x18\x01 \x01(\t\"(\n\x17QueryAliasDenomResponse\x12\r\n\x05\x64\x65nom\x18\x01 \x01(\t2\xeb\x04\n\x05Query\x12y\n\nTokenPairs\x12#.fx.erc20.v1.QueryTokenPairsRequest\x1a$.fx.erc20.v1.QueryTokenPairsResponse\" \x82\xd3\xe4\x93\x02\x1a\x12\x18/fx/erc20/v1/token_pairs\x12~\n\tTokenPair\x12\".fx.erc20.v1.QueryTokenPairRequest\x1a#.fx.erc20.v1.QueryTokenPairResponse\"(\x82\xd3\xe4\x93\x02\"\x12 /fx/erc20/v1/token_pairs/{token}\x12h\n\x06Params\x12\x1f.fx.erc20.v1.QueryParamsRequest\x1a .fx.erc20.v1.QueryParamsResponse\"\x1b\x82\xd3\xe4\x93\x02\x15\x12\x13/fx/erc20/v1/params\x12\x81\x01\n\x0c\x44\x65nomAliases\x12%.fx.erc20.v1.QueryDenomAliasesRequest\x1a&.fx.erc20.v1.QueryDenomAliasesResponse\"\"\x82\xd3\xe4\x93\x02\x1c\x12\x1a/fx/erc20/v1/denom/aliases\x12y\n\nAliasDenom\x12#.fx.erc20.v1.QueryAliasDenomRequest\x1a$.fx.erc20.v1.QueryAliasDenomResponse\" \x82\xd3\xe4\x93\x02\x1a\x12\x18/fx/erc20/v1/alias/denomB,Z*github.com/functionx/fx-core/x/erc20/typesb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'fx.erc20.v1.query_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z*github.com/functionx/fx-core/x/erc20/types'
  _QUERYTOKENPAIRSRESPONSE.fields_by_name['token_pairs']._options = None
  _QUERYTOKENPAIRSRESPONSE.fields_by_name['token_pairs']._serialized_options = b'\310\336\037\000'
  _QUERYTOKENPAIRRESPONSE.fields_by_name['token_pair']._options = None
  _QUERYTOKENPAIRRESPONSE.fields_by_name['token_pair']._serialized_options = b'\310\336\037\000'
  _QUERYPARAMSRESPONSE.fields_by_name['params']._options = None
  _QUERYPARAMSRESPONSE.fields_by_name['params']._serialized_options = b'\310\336\037\000'
  _QUERY.methods_by_name['TokenPairs']._options = None
  _QUERY.methods_by_name['TokenPairs']._serialized_options = b'\202\323\344\223\002\032\022\030/fx/erc20/v1/token_pairs'
  _QUERY.methods_by_name['TokenPair']._options = None
  _QUERY.methods_by_name['TokenPair']._serialized_options = b'\202\323\344\223\002\"\022 /fx/erc20/v1/token_pairs/{token}'
  _QUERY.methods_by_name['Params']._options = None
  _QUERY.methods_by_name['Params']._serialized_options = b'\202\323\344\223\002\025\022\023/fx/erc20/v1/params'
  _QUERY.methods_by_name['DenomAliases']._options = None
  _QUERY.methods_by_name['DenomAliases']._serialized_options = b'\202\323\344\223\002\034\022\032/fx/erc20/v1/denom/aliases'
  _QUERY.methods_by_name['AliasDenom']._options = None
  _QUERY.methods_by_name['AliasDenom']._serialized_options = b'\202\323\344\223\002\032\022\030/fx/erc20/v1/alias/denom'
  _globals['_QUERYTOKENPAIRSREQUEST']._serialized_start=188
  _globals['_QUERYTOKENPAIRSREQUEST']._serialized_end=272
  _globals['_QUERYTOKENPAIRSRESPONSE']._serialized_start=275
  _globals['_QUERYTOKENPAIRSRESPONSE']._serialized_end=412
  _globals['_QUERYTOKENPAIRREQUEST']._serialized_start=414
  _globals['_QUERYTOKENPAIRREQUEST']._serialized_end=452
  _globals['_QUERYTOKENPAIRRESPONSE']._serialized_start=454
  _globals['_QUERYTOKENPAIRRESPONSE']._serialized_end=528
  _globals['_QUERYPARAMSREQUEST']._serialized_start=530
  _globals['_QUERYPARAMSREQUEST']._serialized_end=550
  _globals['_QUERYPARAMSRESPONSE']._serialized_start=552
  _globals['_QUERYPARAMSRESPONSE']._serialized_end=616
  _globals['_QUERYDENOMALIASESREQUEST']._serialized_start=618
  _globals['_QUERYDENOMALIASESREQUEST']._serialized_end=659
  _globals['_QUERYDENOMALIASESRESPONSE']._serialized_start=661
  _globals['_QUERYDENOMALIASESRESPONSE']._serialized_end=705
  _globals['_QUERYALIASDENOMREQUEST']._serialized_start=707
  _globals['_QUERYALIASDENOMREQUEST']._serialized_end=746
  _globals['_QUERYALIASDENOMRESPONSE']._serialized_start=748
  _globals['_QUERYALIASDENOMRESPONSE']._serialized_end=788
  _globals['_QUERY']._serialized_start=791
  _globals['_QUERY']._serialized_end=1410
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ethermint/evm/v1/genesis.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from fxsdk.x.ethermint.evm.v1 import evm_pb2 as ethermint_dot_evm_dot_v1_dot_evm__pb2
from fxsdk.x.gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1e\x65thermint/evm/v1/genesis.proto\x12\x10\x65thermint.evm.v1\x1a\x1a\x65thermint/evm/v1/evm.proto\x1a\x14gogoproto/gogo.proto\"x\n\x0cGenesisState\x12\x38\n\x08\x61\x63\x63ounts\x18\x01 \x03(\x0b\x32 .ethermint.evm.v1.GenesisAccountB\x04\xc8\xde\x1f\x00\x12.\n\x06params\x18\x02 \x01(\x0b\x32\x18.ethermint.evm.v1.ParamsB\x04\xc8\xde\x1f\x00\"j\n\x0eGenesisAccount\x12\x0f\n\x07\x61\x64\x64ress\x18\x01 \x01(\t\x12\x0c\n\x04\x63ode\x18\x02 \x01(\t\x12\x39\n\x07storage\x18\x03 \x03(\x0b\x32\x17.ethermint.evm.v1.StateB\x0f\xc8\xde\x1f\x00\xaa\xdf\x1f\x07StorageB(Z&github.com/evmos/ethermint/x/evm/typesb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ethermint.evm.v1.genesis_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z&github.com/evmos/ethermint/x/evm/types'
  _GENESISSTATE.fields_by_name['accounts']._options = None
  _GENESISSTATE.fields_by_name['accounts']._serialized_options = b'\310\336\037\000'
  _GENESISSTATE.fields_by_name['params']._options = None
  _GENESISSTATE.fields_by_name['params']._serialized_options = b'\310\336\037\000'
  _GENESISACCOUNT.fields_by_name['storage']._options = None
  _GENESISACCOUNT.fields_by_name['storage']._serialized_options = b'\310\336\037\000\252\337\037\007Storage'
  _globals['_GENESISSTATE']._serialized_start=102
  _globals['_GENESISSTATE']._serialized_end=222
  _globals['_GENESISACCOUNT']._serialized_start=224
  _globals['_GENESISACCOUNT']._serialized_end=330
# @@protoc_insertion_point(module_scope)



import hashlib
import ecdsa
import hdwallets
import coincurve

from mnemonic import Mnemonic
from eth_hash.auto import keccak as keccak_256
from google.protobuf.any_pb2 import Any

from fxsdk.x.cosmos.crypto.secp256k1.keys_pb2 import PubKey as secp256k1PubKey
from fxsdk.x.ethermint.crypto.v1.ethsecp256k1.keys_pb2 import PubKey as ethsecp256k1PubKey
from fxsdk.wallet.address import DEFAULT_BECH32_PREFIX, Address

ETH_DERIVATION_PATH = "m/44'/60'/0'/0/0"
ETHSECP256K1_KEY_TYPE = 'eth_secp256k1'
ETHSECP256K1_PUBLICK_TYPE_URL = '/ethermint.crypto.v1.ethsecp256k1.PubKey'

COSMOS_DERIVATION_PATH = "m/44'/118'/0'/0/0"
SECP256K1_KEY_TYPE = 'secp256k1'
SECP256K1_PUBLICK_TYPE_URL = '/cosmos.crypto.secp256k1.PubKey'


class PublicKey:
    def __init__(self, pub_key: bytes, key_type: str = ETHSECP256K1_KEY_TYPE):
        self._pub_key = pub_key
        self._key_type = key_type

    def to_address(self, prefix: str = DEFAULT_BECH32_PREFIX) -> Address:
        if self._key_type == SECP256K1_KEY_TYPE:
            h = hashlib.new("sha256", self._pub_key).digest()
            data = hashlib.new("ripemd160", h).digest()
            if prefix == "0x":
                raise Exception("secp256k1 not support 0x prefix address")
        else:
            pubkey_obj = ecdsa.keys.VerifyingKey.from_string(self._pub_key, curve=ecdsa.SECP256k1)
            data = keccak_256(pubkey_obj.to_string("raw"))[-20:]

        return Address(addr=data, prefix=prefix)

    def to_secp256k1_any(self) -> Any:
        if self._key_type == SECP256K1_KEY_TYPE:
            key = secp256k1PubKey(key=self._pub_key).SerializeToString()
            public_type_url = SECP256K1_PUBLICK_TYPE_URL
            return Any(type_url=public_type_url, value=key)
        else:
            key = ethsecp256k1PubKey(key=self._pub_key).SerializeToString()
            public_type_url = ETHSECP256K1_PUBLICK_TYPE_URL
            return Any(type_url=public_type_url, value=key)

    def to_hex(self) -> str:
        return bytes.hex(self._pub_key)


class PrivateKey:

    def __init__(self, priv_key: bytes, key_type: str = ETHSECP256K1_KEY_TYPE):
        self._priv_key = priv_key
        self._key_type = key_type

    @classmethod
    def from_hex(cls, priv_key: str, key_type: str = ETHSECP256K1_KEY_TYPE) -> 'PrivateKey':
        return cls(priv_key=bytes.fromhex(priv_key), key_type=key_type)

    def with_key_type(self, key_type):
        self._key_type = key_type

    def to_public_key(self) -> PublicKey:
        privkey_obj = ecdsa.SigningKey.from_string(self._priv_key, curve=ecdsa.SECP256k1)
        pubkey_obj = privkey_obj.get_verifying_key()
        return PublicKey(pub_key=pubkey_obj.to_string("compressed"), key_type=self._key_type)

    def to_address(self, prefix: str = DEFAULT_BECH32_PREFIX) -> Address:
        return self.to_public_key().to_address(prefix=prefix)

    def to_hex(self) -> str:
        return bytes.hex(self._priv_key)

    def sign(self, message_bytes: bytes) -> bytes:
        if self._key_type == SECP256K1_KEY_TYPE:
            privkey = ecdsa.SigningKey.from_string(self._priv_key, curve=ecdsa.SECP256k1)
            signature_compact = privkey.sign_deterministic(
                data=message_bytes, hashfunc=hashlib.sha256, sigencode=ecdsa.util.sigencode_string_canonize
            )
        else:
            if len(message_bytes) != 32:
                message_bytes = keccak_256(message_bytes)
            signature_compact = coincurve.keys.PrivateKey(self._priv_key).sign_recoverable(
                message=message_bytes, hasher=None,
            )

        return signature_compact


def generate_mnemonic() -> str:
    return Mnemonic(language="english").generate(strength=256)


def mnemonic_to_privkey(mnemonic: str, passphrase: str = '', path: str = ETH_DERIVATION_PATH,
                        key_type: str = ETHSECP256K1_KEY_TYPE) -> PrivateKey:
    """Get a private key from a mnemonic seed and a derivation path.
    """
    seed_bytes = Mnemonic.to_seed(mnemonic=mnemonic, passphrase=passphrase)
    hd_wallet = hdwallets.BIP32.from_seed(seed_bytes)
    derived_privkey = hd_wallet.get_privkey_from_path(path)
    return PrivateKey(priv_key=derived_privkey, key_type=key_type)


def pubkey_to_address(pubkey: bytes, prefix: str = DEFAULT_BECH32_PREFIX,
                      key_type: str = ETHSECP256K1_KEY_TYPE, ) -> str:
    return PublicKey(pub_key=pubkey, key_type=key_type).to_address(prefix).to_string()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: fx/erc20/v1/erc20.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from fxsdk.x.cosmos.bank.v1beta1 import bank_pb2 as cosmos_dot_bank_dot_v1beta1_dot_bank__pb2
from fxsdk.x.gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x17\x66x/erc20/v1/erc20.proto\x12\x0b\x66x.erc20.v1\x1a\x1e\x63osmos/bank/v1beta1/bank.proto\x1a\x14gogoproto/gogo.proto\"t\n\tTokenPair\x12\x15\n\rerc20_address\x18\x01 \x01(\t\x12\r\n\x05\x64\x65nom\x18\x02 \x01(\t\x12\x0f\n\x07\x65nabled\x18\x03 \x01(\x08\x12*\n\x0e\x63ontract_owner\x18\x04 \x01(\x0e\x32\x12.fx.erc20.v1.Owner:\x04\xe8\xa0\x1f\x01\"w\n\x14RegisterCoinProposal\x12\r\n\x05title\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\x12\x35\n\x08metadata\x18\x03 \x01(\x0b\x32\x1d.cosmos.bank.v1beta1.MetadataB\x04\xc8\xde\x1f\x00:\x04\xe8\xa0\x1f\x00\"h\n\x15RegisterERC20Proposal\x12\r\n\x05title\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\x12\x14\n\x0c\x65rc20address\x18\x03 \x01(\t\x12\x0f\n\x07\x61liases\x18\x04 \x03(\t:\x04\xe8\xa0\x1f\x00\"X\n\x1dToggleTokenConversionProposal\x12\r\n\x05title\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\x12\r\n\x05token\x18\x03 \x01(\t:\x04\xe8\xa0\x1f\x01\"b\n\x18UpdateDenomAliasProposal\x12\r\n\x05title\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\x12\r\n\x05\x64\x65nom\x18\x03 \x01(\t\x12\r\n\x05\x61lias\x18\x04 \x01(\t:\x04\xe8\xa0\x1f\x00*J\n\x05Owner\x12\x15\n\x11OWNER_UNSPECIFIED\x10\x00\x12\x10\n\x0cOWNER_MODULE\x10\x01\x12\x12\n\x0eOWNER_EXTERNAL\x10\x02\x1a\x04\x88\xa3\x1e\x00\x42,Z*github.com/functionx/fx-core/x/erc20/typesb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'fx.erc20.v1.erc20_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z*github.com/functionx/fx-core/x/erc20/types'
  _OWNER._options = None
  _OWNER._serialized_options = b'\210\243\036\000'
  _TOKENPAIR._options = None
  _TOKENPAIR._serialized_options = b'\350\240\037\001'
  _REGISTERCOINPROPOSAL.fields_by_name['metadata']._options = None
  _REGISTERCOINPROPOSAL.fields_by_name['metadata']._serialized_options = b'\310\336\037\000'
  _REGISTERCOINPROPOSAL._options = None
  _REGISTERCOINPROPOSAL._serialized_options = b'\350\240\037\000'
  _REGISTERERC20PROPOSAL._options = None
  _REGISTERERC20PROPOSAL._serialized_options = b'\350\240\037\000'
  _TOGGLETOKENCONVERSIONPROPOSAL._options = None
  _TOGGLETOKENCONVERSIONPROPOSAL._serialized_options = b'\350\240\037\001'
  _UPDATEDENOMALIASPROPOSAL._options = None
  _UPDATEDENOMALIASPROPOSAL._serialized_options = b'\350\240\037\000'
  _globals['_OWNER']._serialized_start=629
  _globals['_OWNER']._serialized_end=703
  _globals['_TOKENPAIR']._serialized_start=94
  _globals['_TOKENPAIR']._serialized_end=210
  _globals['_REGISTERCOINPROPOSAL']._serialized_start=212
  _globals['_REGISTERCOINPROPOSAL']._serialized_end=331
  _globals['_REGISTERERC20PROPOSAL']._serialized_start=333
  _globals['_REGISTERERC20PROPOSAL']._serialized_end=437
  _globals['_TOGGLETOKENCONVERSIONPROPOSAL']._serialized_start=439
  _globals['_TOGGLETOKENCONVERSIONPROPOSAL']._serialized_end=527
  _globals['_UPDATEDENOMALIASPROPOSAL']._serialized_start=529
  _globals['_UPDATEDENOMALIASPROPOSAL']._serialized_end=627
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: fx/gov/v1/params.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from fxsdk.x.cosmos.base.v1beta1 import coin_pb2 as cosmos_dot_base_dot_v1beta1_dot_coin__pb2
from fxsdk.x.cosmos_proto import cosmos_pb2 as cosmos__proto_dot_cosmos__pb2
from fxsdk.x.gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x16\x66x/gov/v1/params.proto\x12\tfx.gov.v1\x1a\x1e\x63osmos/base/v1beta1/coin.proto\x1a\x19\x63osmos_proto/cosmos.proto\x1a\x14gogoproto/gogo.proto\x1a\x1egoogle/protobuf/duration.proto\"\x8f\x04\n\x06Params\x12\x10\n\x08msg_type\x18\x01 \x01(\t\x12M\n\x0bmin_deposit\x18\x02 \x03(\x0b\x32\x19.cosmos.base.v1beta1.CoinB\x1d\xc8\xde\x1f\x00\xea\xde\x1f\x15min_deposit,omitempty\x12]\n\x13min_initial_deposit\x18\x03 \x01(\x0b\x32\x19.cosmos.base.v1beta1.CoinB%\xc8\xde\x1f\x00\xea\xde\x1f\x1dmin_initial_deposit,omitempty\x12\x36\n\rvoting_period\x18\x04 \x01(\x0b\x32\x19.google.protobuf.DurationB\x04\x98\xdf\x1f\x01\x12\x32\n\x06quorum\x18\x05 \x01(\tB\"\xea\xde\x1f\x10quorum,omitempty\xd2\xb4-\ncosmos.Dec\x12[\n\x12max_deposit_period\x18\x06 \x01(\x0b\x32\x19.google.protobuf.DurationB$\xea\xde\x1f\x1cmax_deposit_period,omitempty\x98\xdf\x1f\x01\x12\x38\n\tthreshold\x18\x07 \x01(\tB%\xea\xde\x1f\x13threshold,omitempty\xd2\xb4-\ncosmos.Dec\x12\x42\n\x0eveto_threshold\x18\x08 \x01(\tB*\xea\xde\x1f\x18veto_threshold,omitempty\xd2\xb4-\ncosmos.Dec\"\xac\x01\n\tEGFParams\x12\x61\n\x15\x65gf_deposit_threshold\x18\x01 \x01(\x0b\x32\x19.cosmos.base.v1beta1.CoinB\'\xc8\xde\x1f\x00\xea\xde\x1f\x1f\x65gf_deposit_threshold,omitempty\x12<\n\x0b\x63laim_ratio\x18\x02 \x01(\tB\'\xea\xde\x1f\x15\x63laim_ratio,omitempty\xd2\xb4-\ncosmos.DecB*Z(github.com/functionx/fx-core/x/gov/typesb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'fx.gov.v1.params_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z(github.com/functionx/fx-core/x/gov/types'
  _PARAMS.fields_by_name['min_deposit']._options = None
  _PARAMS.fields_by_name['min_deposit']._serialized_options = b'\310\336\037\000\352\336\037\025min_deposit,omitempty'
  _PARAMS.fields_by_name['min_initial_deposit']._options = None
  _PARAMS.fields_by_name['min_initial_deposit']._serialized_options = b'\310\336\037\000\352\336\037\035min_initial_deposit,omitempty'
  _PARAMS.fields_by_name['voting_period']._options = None
  _PARAMS.fields_by_name['voting_period']._serialized_options = b'\230\337\037\001'
  _PARAMS.fields_by_name['quorum']._options = None
  _PARAMS.fields_by_name['quorum']._serialized_options = b'\352\336\037\020quorum,omitempty\322\264-\ncosmos.Dec'
  _PARAMS.fields_by_name['max_deposit_period']._options = None
  _PARAMS.fields_by_name['max_deposit_period']._serialized_options = b'\352\336\037\034max_deposit_period,omitempty\230\337\037\001'
  _PARAMS.fields_by_name['threshold']._options = None
  _PARAMS.fields_by_name['threshold']._serialized_options = b'\352\336\037\023threshold,omitempty\322\264-\ncosmos.Dec'
  _PARAMS.fields_by_name['veto_threshold']._options = None
  _PARAMS.fields_by_name['veto_threshold']._serialized_options = b'\352\336\037\030veto_threshold,omitempty\322\264-\ncosmos.Dec'
  _EGFPARAMS.fields_by_name['egf_deposit_threshold']._options = None
  _EGFPARAMS.fields_by_name['egf_deposit_threshold']._serialized_options = b'\310\336\037\000\352\336\037\037egf_deposit_threshold,omitempty'
  _EGFPARAMS.fields_by_name['claim_ratio']._options = None
  _EGFPARAMS.fields_by_name['claim_ratio']._serialized_options = b'\352\336\037\025claim_ratio,omitempty\322\264-\ncosmos.Dec'
  _globals['_PARAMS']._serialized_start=151
  _globals['_PARAMS']._serialized_end=678
  _globals['_EGFPARAMS']._serialized_start=681
  _globals['_EGFPARAMS']._serialized_end=853
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: fx/evm/v1/tx.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from fxsdk.x.cosmos.msg.v1 import msg_pb2 as cosmos_dot_msg_dot_v1_dot_msg__pb2
from fxsdk.x.cosmos_proto import cosmos_pb2 as cosmos__proto_dot_cosmos__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x12\x66x/evm/v1/tx.proto\x12\tfx.evm.v1\x1a\x17\x63osmos/msg/v1/msg.proto\x1a\x19\x63osmos_proto/cosmos.proto\"v\n\x0fMsgCallContract\x12+\n\tauthority\x18\x01 \x01(\tB\x18\xd2\xb4-\x14\x63osmos.AddressString\x12\x18\n\x10\x63ontract_address\x18\x02 \x01(\t\x12\x0c\n\x04\x64\x61ta\x18\x03 \x01(\t:\x0e\x82\xe7\xb0*\tauthority\"\x19\n\x17MsgCallContractResponse2U\n\x03Msg\x12N\n\x0c\x43\x61llContract\x12\x1a.fx.evm.v1.MsgCallContract\x1a\".fx.evm.v1.MsgCallContractResponseB*Z(github.com/functionx/fx-core/x/evm/typesb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'fx.evm.v1.tx_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z(github.com/functionx/fx-core/x/evm/types'
  _MSGCALLCONTRACT.fields_by_name['authority']._options = None
  _MSGCALLCONTRACT.fields_by_name['authority']._serialized_options = b'\322\264-\024cosmos.AddressString'
  _MSGCALLCONTRACT._options = None
  _MSGCALLCONTRACT._serialized_options = b'\202\347\260*\tauthority'
  _globals['_MSGCALLCONTRACT']._serialized_start=85
  _globals['_MSGCALLCONTRACT']._serialized_end=203
  _globals['_MSGCALLCONTRACTRESPONSE']._serialized_start=205
  _globals['_MSGCALLCONTRACTRESPONSE']._serialized_end=230
  _globals['_MSG']._serialized_start=232
  _globals['_MSG']._serialized_end=317
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: cosmos/crisis/v1beta1/tx.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from fxsdk.x.gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2
from fxsdk.x.cosmos_proto import cosmos_pb2 as cosmos__proto_dot_cosmos__pb2
from fxsdk.x.cosmos.msg.v1 import msg_pb2 as cosmos_dot_msg_dot_v1_dot_msg__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1e\x63osmos/crisis/v1beta1/tx.proto\x12\x15\x63osmos.crisis.v1beta1\x1a\x14gogoproto/gogo.proto\x1a\x19\x63osmos_proto/cosmos.proto\x1a\x17\x63osmos/msg/v1/msg.proto\"\x8b\x01\n\x12MsgVerifyInvariant\x12(\n\x06sender\x18\x01 \x01(\tB\x18\xd2\xb4-\x14\x63osmos.AddressString\x12\x1d\n\x15invariant_module_name\x18\x02 \x01(\t\x12\x17\n\x0finvariant_route\x18\x03 \x01(\t:\x13\x88\xa0\x1f\x00\xe8\xa0\x1f\x00\x82\xe7\xb0*\x06sender\"\x1c\n\x1aMsgVerifyInvariantResponse2v\n\x03Msg\x12o\n\x0fVerifyInvariant\x12).cosmos.crisis.v1beta1.MsgVerifyInvariant\x1a\x31.cosmos.crisis.v1beta1.MsgVerifyInvariantResponseB-Z+github.com/cosmos/cosmos-sdk/x/crisis/typesb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'cosmos.crisis.v1beta1.tx_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z+github.com/cosmos/cosmos-sdk/x/crisis/types'
  _MSGVERIFYINVARIANT.fields_by_name['sender']._options = None
  _MSGVERIFYINVARIANT.fields_by_name['sender']._serialized_options = b'\322\264-\024cosmos.AddressString'
  _MSGVERIFYINVARIANT._options = None
  _MSGVERIFYINVARIANT._serialized_options = b'\210\240\037\000\350\240\037\000\202\347\260*\006sender'
  _globals['_MSGVERIFYINVARIANT']._serialized_start=132
  _globals['_MSGVERIFYINVARIANT']._serialized_end=271
  _globals['_MSGVERIFYINVARIANTRESPONSE']._serialized_start=273
  _globals['_MSGVERIFYINVARIANTRESPONSE']._serialized_end=301
  _globals['_MSG']._serialized_start=303
  _globals['_MSG']._serialized_end=421
# @@protoc_insertion_point(module_scope)



import time
from typing import Dict

from fxsdk.client.http import HttpRpcClient, RPCException
from fxsdk.msg import new_tx_from_base64
from fxsdk.x.cosmos.tx.v1beta1.tx_pb2 import Tx


class ScanException(Exception):
    pass


class ScanBlock(HttpRpcClient):

    def start(self, block_interval_ms=500, start_block_height: int = -1):
        if start_block_height <= 0:
            status = self.get_status()
            if status.get('sync_info', {}).get('catching_up', True):
                raise ScanException("node is catching up")
            latest_block_height = status.get('sync_info', {}).get('latest_block_height', 1)
            start_block_height = int(latest_block_height)
        while True:
            try:
                block_result = self.get_block_results(start_block_height)
                txs_results = block_result.get('txs_results', [])
                txs_results = txs_results if txs_results else []
                txs = []
                if len(txs_results) > 0:
                    block = self.get_block(start_block_height)
                    base64_txs = block.get('block', {}).get('data', {}).get('txs', [])
                    for base64_tx in base64_txs:
                        tx = new_tx_from_base64(base64_tx)
                        txs.append(tx)
                block_events = block_result.get('finalize_block_events', [])
                block_height = block_result.get('height', 0)
                self.process_new_block(block_height, block_events, txs_results, txs)
                start_block_height += 1
            except RPCException as e:
                self._log.debug(f"rpc exception: {e}")
                continue
            time.sleep(block_interval_ms / 1000)

    def process_new_block(self, block_height: int, block_events: [Dict], txs_result: [Dict], txs: [Tx]):
        self._log.info(
            f"new block: height={block_height}, txs={len(txs)}, txs_result={len(txs_result)} events={len(block_events)}")



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: fx/dex/v1/tx.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from fxsdk.x.gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2
from fxsdk.x.fx.dex.v1 import order_pb2 as fx_dot_dex_dot_v1_dot_order__pb2
from fxsdk.x.cosmos.base.v1beta1 import coin_pb2 as cosmos_dot_base_dot_v1beta1_dot_coin__pb2
from fxsdk.x.cosmos.msg.v1 import msg_pb2 as cosmos_dot_msg_dot_v1_dot_msg__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x12\x66x/dex/v1/tx.proto\x12\tfx.dex.v1\x1a\x14gogoproto/gogo.proto\x1a\x15\x66x/dex/v1/order.proto\x1a\x1e\x63osmos/base/v1beta1/coin.proto\x1a\x17\x63osmos/msg/v1/msg.proto\"\xe7\x01\n\x0eMsgCreateOrder\x12\r\n\x05owner\x18\x01 \x01(\t\x12\x0f\n\x07pair_id\x18\x02 \x01(\t\x12\'\n\tdirection\x18\x03 \x01(\x0e\x32\x14.fx.dex.v1.Direction\x12\x32\n\x05price\x18\x04 \x01(\tB#\xc8\xde\x1f\x00\xda\xde\x1f\x1b\x63osmossdk.io/math.LegacyDec\x12:\n\rbase_quantity\x18\x05 \x01(\tB#\xc8\xde\x1f\x00\xda\xde\x1f\x1b\x63osmossdk.io/math.LegacyDec\x12\x10\n\x08leverage\x18\x06 \x01(\x03:\n\x82\xe7\xb0*\x05owner\"*\n\x16MsgCreateOrderResponse\x12\x10\n\x08order_id\x18\x01 \x01(\t\"=\n\x0eMsgCancelOrder\x12\r\n\x05owner\x18\x01 \x01(\t\x12\x10\n\x08order_id\x18\x02 \x01(\t:\n\x82\xe7\xb0*\x05owner\"\x18\n\x16MsgCancelOrderResponse\"\xa4\x01\n\x0cMsgAddMargin\x12\r\n\x05owner\x18\x01 \x01(\t\x12\x0f\n\x07pair_id\x18\x02 \x01(\t\x12\x33\n\x0bposition_id\x18\x03 \x01(\tB\x1e\xc8\xde\x1f\x00\xda\xde\x1f\x16\x63osmossdk.io/math.Uint\x12\x33\n\x06margin\x18\x04 \x01(\tB#\xc8\xde\x1f\x00\xda\xde\x1f\x1b\x63osmossdk.io/math.LegacyDec:\n\x82\xe7\xb0*\x05owner\"\x12\n\x10MsgAddMarginResp\"\x8d\x02\n\x10MsgClosePosition\x12\r\n\x05owner\x18\x01 \x01(\t\x12\x0f\n\x07pair_id\x18\x02 \x01(\t\x12\x33\n\x0bposition_id\x18\x03 \x01(\tB\x1e\xc8\xde\x1f\x00\xda\xde\x1f\x16\x63osmossdk.io/math.Uint\x12\x32\n\x05price\x18\x04 \x01(\tB#\xc8\xde\x1f\x00\xda\xde\x1f\x1b\x63osmossdk.io/math.LegacyDec\x12:\n\rbase_quantity\x18\x05 \x01(\tB#\xc8\xde\x1f\x00\xda\xde\x1f\x1b\x63osmossdk.io/math.LegacyDec\x12\x12\n\nfull_close\x18\x06 \x01(\x08\x12\x14\n\x0cmarket_close\x18\x07 \x01(\x08:\n\x82\xe7\xb0*\x05owner\"(\n\x14MsgClosePositionResp\x12\x10\n\x08order_id\x18\x01 \x01(\t\"s\n\x16MsgLiquidationPosition\x12\x12\n\nliquidator\x18\x01 \x01(\t\x12\x34\n\x0cposition_ids\x18\x02 \x03(\tB\x1e\xc8\xde\x1f\x00\xda\xde\x1f\x16\x63osmossdk.io/math.Uint:\x0f\x82\xe7\xb0*\nliquidator\"\x1c\n\x1aMsgLiquidationPositionResp\"\x90\x01\n\x0eMsgFundDexPool\x12[\n\x06\x61mount\x18\x01 \x03(\x0b\x32\x19.cosmos.base.v1beta1.CoinB0\xc8\xde\x1f\x00\xaa\xdf\x1f(github.com/cosmos/cosmos-sdk/types.Coins\x12\x11\n\tdepositor\x18\x02 \x01(\t:\x0e\x82\xe7\xb0*\tdepositor\"\x14\n\x12MsgFundDexPoolResp2\xdb\x03\n\x03Msg\x12K\n\x0b\x43reateOrder\x12\x19.fx.dex.v1.MsgCreateOrder\x1a!.fx.dex.v1.MsgCreateOrderResponse\x12K\n\x0b\x43\x61ncelOrder\x12\x19.fx.dex.v1.MsgCancelOrder\x1a!.fx.dex.v1.MsgCancelOrderResponse\x12\x41\n\tAddMargin\x12\x17.fx.dex.v1.MsgAddMargin\x1a\x1b.fx.dex.v1.MsgAddMarginResp\x12M\n\rClosePosition\x12\x1b.fx.dex.v1.MsgClosePosition\x1a\x1f.fx.dex.v1.MsgClosePositionResp\x12_\n\x13LiquidationPosition\x12!.fx.dex.v1.MsgLiquidationPosition\x1a%.fx.dex.v1.MsgLiquidationPositionResp\x12G\n\x0b\x46undDexPool\x12\x19.fx.dex.v1.MsgFundDexPool\x1a\x1d.fx.dex.v1.MsgFundDexPoolRespB-Z+github.com/marginxio/marginx/x/dex/v1/typesb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'fx.dex.v1.tx_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z+github.com/marginxio/marginx/x/dex/v1/types'
  _MSGCREATEORDER.fields_by_name['price']._options = None
  _MSGCREATEORDER.fields_by_name['price']._serialized_options = b'\310\336\037\000\332\336\037\033cosmossdk.io/math.LegacyDec'
  _MSGCREATEORDER.fields_by_name['base_quantity']._options = None
  _MSGCREATEORDER.fields_by_name['base_quantity']._serialized_options = b'\310\336\037\000\332\336\037\033cosmossdk.io/math.LegacyDec'
  _MSGCREATEORDER._options = None
  _MSGCREATEORDER._serialized_options = b'\202\347\260*\005owner'
  _MSGCANCELORDER._options = None
  _MSGCANCELORDER._serialized_options = b'\202\347\260*\005owner'
  _MSGADDMARGIN.fields_by_name['position_id']._options = None
  _MSGADDMARGIN.fields_by_name['position_id']._serialized_options = b'\310\336\037\000\332\336\037\026cosmossdk.io/math.Uint'
  _MSGADDMARGIN.fields_by_name['margin']._options = None
  _MSGADDMARGIN.fields_by_name['margin']._serialized_options = b'\310\336\037\000\332\336\037\033cosmossdk.io/math.LegacyDec'
  _MSGADDMARGIN._options = None
  _MSGADDMARGIN._serialized_options = b'\202\347\260*\005owner'
  _MSGCLOSEPOSITION.fields_by_name['position_id']._options = None
  _MSGCLOSEPOSITION.fields_by_name['position_id']._serialized_options = b'\310\336\037\000\332\336\037\026cosmossdk.io/math.Uint'
  _MSGCLOSEPOSITION.fields_by_name['price']._options = None
  _MSGCLOSEPOSITION.fields_by_name['price']._serialized_options = b'\310\336\037\000\332\336\037\033cosmossdk.io/math.LegacyDec'
  _MSGCLOSEPOSITION.fields_by_name['base_quantity']._options = None
  _MSGCLOSEPOSITION.fields_by_name['base_quantity']._serialized_options = b'\310\336\037\000\332\336\037\033cosmossdk.io/math.LegacyDec'
  _MSGCLOSEPOSITION._options = None
  _MSGCLOSEPOSITION._serialized_options = b'\202\347\260*\005owner'
  _MSGLIQUIDATIONPOSITION.fields_by_name['position_ids']._options = None
  _MSGLIQUIDATIONPOSITION.fields_by_name['position_ids']._serialized_options = b'\310\336\037\000\332\336\037\026cosmossdk.io/math.Uint'
  _MSGLIQUIDATIONPOSITION._options = None
  _MSGLIQUIDATIONPOSITION._serialized_options = b'\202\347\260*\nliquidator'
  _MSGFUNDDEXPOOL.fields_by_name['amount']._options = None
  _MSGFUNDDEXPOOL.fields_by_name['amount']._serialized_options = b'\310\336\037\000\252\337\037(github.com/cosmos/cosmos-sdk/types.Coins'
  _MSGFUNDDEXPOOL._options = None
  _MSGFUNDDEXPOOL._serialized_options = b'\202\347\260*\tdepositor'
  _globals['_MSGCREATEORDER']._serialized_start=136
  _globals['_MSGCREATEORDER']._serialized_end=367
  _globals['_MSGCREATEORDERRESPONSE']._serialized_start=369
  _globals['_MSGCREATEORDERRESPONSE']._serialized_end=411
  _globals['_MSGCANCELORDER']._serialized_start=413
  _globals['_MSGCANCELORDER']._serialized_end=474
  _globals['_MSGCANCELORDERRESPONSE']._serialized_start=476
  _globals['_MSGCANCELORDERRESPONSE']._serialized_end=500
  _globals['_MSGADDMARGIN']._serialized_start=503
  _globals['_MSGADDMARGIN']._serialized_end=667
  _globals['_MSGADDMARGINRESP']._serialized_start=669
  _globals['_MSGADDMARGINRESP']._serialized_end=687
  _globals['_MSGCLOSEPOSITION']._serialized_start=690
  _globals['_MSGCLOSEPOSITION']._serialized_end=959
  _globals['_MSGCLOSEPOSITIONRESP']._serialized_start=961
  _globals['_MSGCLOSEPOSITIONRESP']._serialized_end=1001
  _globals['_MSGLIQUIDATIONPOSITION']._serialized_start=1003
  _globals['_MSGLIQUIDATIONPOSITION']._serialized_end=1118
  _globals['_MSGLIQUIDATIONPOSITIONRESP']._serialized_start=1120
  _globals['_MSGLIQUIDATIONPOSITIONRESP']._serialized_end=1148
  _globals['_MSGFUNDDEXPOOL']._serialized_start=1151
  _globals['_MSGFUNDDEXPOOL']._serialized_end=1295
  _globals['_MSGFUNDDEXPOOLRESP']._serialized_start=1297
  _globals['_MSGFUNDDEXPOOLRESP']._serialized_end=1317
  _globals['_MSG']._serialized_start=1320
  _globals['_MSG']._serialized_end=1795
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: cosmos/slashing/v1beta1/genesis.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from fxsdk.x.gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2
from fxsdk.x.cosmos.slashing.v1beta1 import slashing_pb2 as cosmos_dot_slashing_dot_v1beta1_dot_slashing__pb2
from fxsdk.x.cosmos_proto import cosmos_pb2 as cosmos__proto_dot_cosmos__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n%cosmos/slashing/v1beta1/genesis.proto\x12\x17\x63osmos.slashing.v1beta1\x1a\x14gogoproto/gogo.proto\x1a&cosmos/slashing/v1beta1/slashing.proto\x1a\x19\x63osmos_proto/cosmos.proto\"\xd5\x01\n\x0cGenesisState\x12\x35\n\x06params\x18\x01 \x01(\x0b\x32\x1f.cosmos.slashing.v1beta1.ParamsB\x04\xc8\xde\x1f\x00\x12\x41\n\rsigning_infos\x18\x02 \x03(\x0b\x32$.cosmos.slashing.v1beta1.SigningInfoB\x04\xc8\xde\x1f\x00\x12K\n\rmissed_blocks\x18\x03 \x03(\x0b\x32..cosmos.slashing.v1beta1.ValidatorMissedBlocksB\x04\xc8\xde\x1f\x00\"\x8d\x01\n\x0bSigningInfo\x12)\n\x07\x61\x64\x64ress\x18\x01 \x01(\tB\x18\xd2\xb4-\x14\x63osmos.AddressString\x12S\n\x16validator_signing_info\x18\x02 \x01(\x0b\x32-.cosmos.slashing.v1beta1.ValidatorSigningInfoB\x04\xc8\xde\x1f\x00\"\x85\x01\n\x15ValidatorMissedBlocks\x12)\n\x07\x61\x64\x64ress\x18\x01 \x01(\tB\x18\xd2\xb4-\x14\x63osmos.AddressString\x12\x41\n\rmissed_blocks\x18\x02 \x03(\x0b\x32$.cosmos.slashing.v1beta1.MissedBlockB\x04\xc8\xde\x1f\x00\",\n\x0bMissedBlock\x12\r\n\x05index\x18\x01 \x01(\x03\x12\x0e\n\x06missed\x18\x02 \x01(\x08\x42/Z-github.com/cosmos/cosmos-sdk/x/slashing/typesb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'cosmos.slashing.v1beta1.genesis_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z-github.com/cosmos/cosmos-sdk/x/slashing/types'
  _GENESISSTATE.fields_by_name['params']._options = None
  _GENESISSTATE.fields_by_name['params']._serialized_options = b'\310\336\037\000'
  _GENESISSTATE.fields_by_name['signing_infos']._options = None
  _GENESISSTATE.fields_by_name['signing_infos']._serialized_options = b'\310\336\037\000'
  _GENESISSTATE.fields_by_name['missed_blocks']._options = None
  _GENESISSTATE.fields_by_name['missed_blocks']._serialized_options = b'\310\336\037\000'
  _SIGNINGINFO.fields_by_name['address']._options = None
  _SIGNINGINFO.fields_by_name['address']._serialized_options = b'\322\264-\024cosmos.AddressString'
  _SIGNINGINFO.fields_by_name['validator_signing_info']._options = None
  _SIGNINGINFO.fields_by_name['validator_signing_info']._serialized_options = b'\310\336\037\000'
  _VALIDATORMISSEDBLOCKS.fields_by_name['address']._options = None
  _VALIDATORMISSEDBLOCKS.fields_by_name['address']._serialized_options = b'\322\264-\024cosmos.AddressString'
  _VALIDATORMISSEDBLOCKS.fields_by_name['missed_blocks']._options = None
  _VALIDATORMISSEDBLOCKS.fields_by_name['missed_blocks']._serialized_options = b'\310\336\037\000'
  _globals['_GENESISSTATE']._serialized_start=156
  _globals['_GENESISSTATE']._serialized_end=369
  _globals['_SIGNINGINFO']._serialized_start=372
  _globals['_SIGNINGINFO']._serialized_end=513
  _globals['_VALIDATORMISSEDBLOCKS']._serialized_start=516
  _globals['_VALIDATORMISSEDBLOCKS']._serialized_end=649
  _globals['_MISSEDBLOCK']._serialized_start=651
  _globals['_MISSEDBLOCK']._serialized_end=695
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: fx/crosschain/v1/tx.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from fxsdk.x.cosmos.base.v1beta1 import coin_pb2 as cosmos_dot_base_dot_v1beta1_dot_coin__pb2
from fxsdk.x.cosmos.msg.v1 import msg_pb2 as cosmos_dot_msg_dot_v1_dot_msg__pb2
from fxsdk.x.cosmos_proto import cosmos_pb2 as cosmos__proto_dot_cosmos__pb2
from fxsdk.x.fx.crosschain.v1 import types_pb2 as fx_dot_crosschain_dot_v1_dot_types__pb2
from fxsdk.x.gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x19\x66x/crosschain/v1/tx.proto\x12\x18\x66x.gravity.crosschain.v1\x1a\x1e\x63osmos/base/v1beta1/coin.proto\x1a\x17\x63osmos/msg/v1/msg.proto\x1a\x19\x63osmos_proto/cosmos.proto\x1a\x1c\x66x/crosschain/v1/types.proto\x1a\x14gogoproto/gogo.proto\"\xc5\x01\n\x0fMsgBondedOracle\x12\x12\n\nchain_name\x18\x01 \x01(\t\x12\x16\n\x0eoracle_address\x18\x02 \x01(\t\x12\x17\n\x0f\x62ridger_address\x18\x03 \x01(\t\x12\x18\n\x10\x65xternal_address\x18\x04 \x01(\t\x12\x19\n\x11validator_address\x18\x05 \x01(\t\x12\x38\n\x0f\x64\x65legate_amount\x18\x06 \x01(\x0b\x32\x19.cosmos.base.v1beta1.CoinB\x04\xc8\xde\x1f\x00\"\x19\n\x17MsgBondedOracleResponse\"m\n\x0eMsgAddDelegate\x12\x12\n\nchain_name\x18\x01 \x01(\t\x12\x16\n\x0eoracle_address\x18\x02 \x01(\t\x12/\n\x06\x61mount\x18\x03 \x01(\x0b\x32\x19.cosmos.base.v1beta1.CoinB\x04\xc8\xde\x1f\x00\"\x18\n\x16MsgAddDelegateResponse\"V\n\rMsgReDelegate\x12\x12\n\nchain_name\x18\x01 \x01(\t\x12\x16\n\x0eoracle_address\x18\x02 \x01(\t\x12\x19\n\x11validator_address\x18\x03 \x01(\t\"\x17\n\x15MsgReDelegateResponse\"U\n\x0eMsgEditBridger\x12\x12\n\nchain_name\x18\x01 \x01(\t\x12\x16\n\x0eoracle_address\x18\x02 \x01(\t\x12\x17\n\x0f\x62ridger_address\x18\x03 \x01(\t\"\x18\n\x16MsgEditBridgerResponse\"?\n\x11MsgUnbondedOracle\x12\x12\n\nchain_name\x18\x01 \x01(\t\x12\x16\n\x0eoracle_address\x18\x02 \x01(\t\"\x1b\n\x19MsgUnbondedOracleResponse\"?\n\x11MsgWithdrawReward\x12\x12\n\nchain_name\x18\x01 \x01(\t\x12\x16\n\x0eoracle_address\x18\x02 \x01(\t\"\x1b\n\x19MsgWithdrawRewardResponse\"~\n\x13MsgOracleSetConfirm\x12\r\n\x05nonce\x18\x01 \x01(\x04\x12\x17\n\x0f\x62ridger_address\x18\x02 \x01(\t\x12\x18\n\x10\x65xternal_address\x18\x03 \x01(\t\x12\x11\n\tsignature\x18\x04 \x01(\t\x12\x12\n\nchain_name\x18\x05 \x01(\t\"\x1d\n\x1bMsgOracleSetConfirmResponse\"\xce\x01\n\x18MsgOracleSetUpdatedClaim\x12\x13\n\x0b\x65vent_nonce\x18\x01 \x01(\x04\x12\x14\n\x0c\x62lock_height\x18\x02 \x01(\x04\x12\x18\n\x10oracle_set_nonce\x18\x03 \x01(\x04\x12@\n\x07members\x18\x04 \x03(\x0b\x32).fx.gravity.crosschain.v1.BridgeValidatorB\x04\xc8\xde\x1f\x00\x12\x17\n\x0f\x62ridger_address\x18\x06 \x01(\t\x12\x12\n\nchain_name\x18\x07 \x01(\t\"\"\n MsgOracleSetUpdatedClaimResponse\"\xf8\x01\n\x10MsgSendToFxClaim\x12\x13\n\x0b\x65vent_nonce\x18\x01 \x01(\x04\x12\x14\n\x0c\x62lock_height\x18\x02 \x01(\x04\x12\x16\n\x0etoken_contract\x18\x03 \x01(\t\x12>\n\x06\x61mount\x18\x04 \x01(\tB.\xc8\xde\x1f\x00\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Int\x12\x0e\n\x06sender\x18\x05 \x01(\t\x12\x10\n\x08receiver\x18\x06 \x01(\t\x12\x12\n\ntarget_ibc\x18\x07 \x01(\t\x12\x17\n\x0f\x62ridger_address\x18\x08 \x01(\t\x12\x12\n\nchain_name\x18\t \x01(\t\"\x1a\n\x18MsgSendToFxClaimResponse\"\xab\x01\n\x11MsgSendToExternal\x12\x0e\n\x06sender\x18\x01 \x01(\t\x12\x0c\n\x04\x64\x65st\x18\x02 \x01(\t\x12/\n\x06\x61mount\x18\x03 \x01(\x0b\x32\x19.cosmos.base.v1beta1.CoinB\x04\xc8\xde\x1f\x00\x12\x33\n\nbridge_fee\x18\x04 \x01(\x0b\x32\x19.cosmos.base.v1beta1.CoinB\x04\xc8\xde\x1f\x00\x12\x12\n\nchain_name\x18\x05 \x01(\t\"3\n\x19MsgSendToExternalResponse\x12\x16\n\x0eoutgoing_tx_id\x18\x01 \x01(\x04\"U\n\x17MsgCancelSendToExternal\x12\x16\n\x0etransaction_id\x18\x01 \x01(\x04\x12\x0e\n\x06sender\x18\x02 \x01(\t\x12\x12\n\nchain_name\x18\x03 \x01(\t\"!\n\x1fMsgCancelSendToExternalResponse\"\xe0\x01\n\x0fMsgRequestBatch\x12\x0e\n\x06sender\x18\x01 \x01(\t\x12\r\n\x05\x64\x65nom\x18\x02 \x01(\t\x12\x43\n\x0bminimum_fee\x18\x03 \x01(\tB.\xc8\xde\x1f\x00\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Int\x12\x13\n\x0b\x66\x65\x65_receive\x18\x04 \x01(\t\x12\x12\n\nchain_name\x18\x05 \x01(\t\x12@\n\x08\x62\x61se_fee\x18\x06 \x01(\tB.\xc8\xde\x1f\x00\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Int\".\n\x17MsgRequestBatchResponse\x12\x13\n\x0b\x62\x61tch_nonce\x18\x01 \x01(\x04\"\x92\x01\n\x0fMsgConfirmBatch\x12\r\n\x05nonce\x18\x01 \x01(\x04\x12\x16\n\x0etoken_contract\x18\x02 \x01(\t\x12\x17\n\x0f\x62ridger_address\x18\x03 \x01(\t\x12\x18\n\x10\x65xternal_address\x18\x04 \x01(\t\x12\x11\n\tsignature\x18\x05 \x01(\t\x12\x12\n\nchain_name\x18\x06 \x01(\t\"\x19\n\x17MsgConfirmBatchResponse\"\x9d\x01\n\x16MsgSendToExternalClaim\x12\x13\n\x0b\x65vent_nonce\x18\x01 \x01(\x04\x12\x14\n\x0c\x62lock_height\x18\x02 \x01(\x04\x12\x13\n\x0b\x62\x61tch_nonce\x18\x03 \x01(\x04\x12\x16\n\x0etoken_contract\x18\x04 \x01(\t\x12\x17\n\x0f\x62ridger_address\x18\x05 \x01(\t\x12\x12\n\nchain_name\x18\x06 \x01(\t\" \n\x1eMsgSendToExternalClaimResponse\"\xca\x01\n\x13MsgBridgeTokenClaim\x12\x13\n\x0b\x65vent_nonce\x18\x01 \x01(\x04\x12\x14\n\x0c\x62lock_height\x18\x02 \x01(\x04\x12\x16\n\x0etoken_contract\x18\x03 \x01(\t\x12\x0c\n\x04name\x18\x04 \x01(\t\x12\x0e\n\x06symbol\x18\x05 \x01(\t\x12\x10\n\x08\x64\x65\x63imals\x18\x06 \x01(\x04\x12\x17\n\x0f\x62ridger_address\x18\x07 \x01(\t\x12\x13\n\x0b\x63hannel_ibc\x18\x08 \x01(\t\x12\x12\n\nchain_name\x18\t \x01(\t\"\x1d\n\x1bMsgBridgeTokenClaimResponse\"\xac\x01\n\x19MsgSetOrchestratorAddress\x12\x16\n\x0eoracle_address\x18\x01 \x01(\t\x12\x17\n\x0f\x62ridger_address\x18\x02 \x01(\t\x12\x18\n\x10\x65xternal_address\x18\x03 \x01(\t\x12\x30\n\x07\x64\x65posit\x18\x04 \x01(\x0b\x32\x19.cosmos.base.v1beta1.CoinB\x04\xc8\xde\x1f\x00\x12\x12\n\nchain_name\x18\x05 \x01(\t\"r\n\x13MsgAddOracleDeposit\x12\x16\n\x0eoracle_address\x18\x01 \x01(\t\x12/\n\x06\x61mount\x18\x02 \x01(\x0b\x32\x19.cosmos.base.v1beta1.CoinB\x04\xc8\xde\x1f\x00\x12\x12\n\nchain_name\x18\x03 \x01(\t\"\x9a\x01\n\x0fMsgUpdateParams\x12\x12\n\nchain_name\x18\x01 \x01(\t\x12+\n\tauthority\x18\x02 \x01(\tB\x18\xd2\xb4-\x14\x63osmos.AddressString\x12\x36\n\x06params\x18\x03 \x01(\x0b\x32 .fx.gravity.crosschain.v1.ParamsB\x04\xc8\xde\x1f\x00:\x0e\x82\xe7\xb0*\tauthority\"\x19\n\x17MsgUpdateParamsResponse\"\x8b\x01\n\x14MsgIncreaseBridgeFee\x12\x12\n\nchain_name\x18\x01 \x01(\t\x12\x16\n\x0etransaction_id\x18\x02 \x01(\x04\x12\x0e\n\x06sender\x18\x03 \x01(\t\x12\x37\n\x0e\x61\x64\x64_bridge_fee\x18\x04 \x01(\x0b\x32\x19.cosmos.base.v1beta1.CoinB\x04\xc8\xde\x1f\x00\"\x1e\n\x1cMsgIncreaseBridgeFeeResponse\"y\n\x15MsgUpdateChainOracles\x12\x12\n\nchain_name\x18\x01 \x01(\t\x12+\n\tauthority\x18\x02 \x01(\tB\x18\xd2\xb4-\x14\x63osmos.AddressString\x12\x0f\n\x07oracles\x18\x03 \x03(\t:\x0e\x82\xe7\xb0*\tauthority\"\x1f\n\x1dMsgUpdateChainOraclesResponse2\xcd\x10\n\x03Msg\x12l\n\x0c\x42ondedOracle\x12).fx.gravity.crosschain.v1.MsgBondedOracle\x1a\x31.fx.gravity.crosschain.v1.MsgBondedOracleResponse\x12i\n\x0b\x41\x64\x64\x44\x65legate\x12(.fx.gravity.crosschain.v1.MsgAddDelegate\x1a\x30.fx.gravity.crosschain.v1.MsgAddDelegateResponse\x12\x66\n\nReDelegate\x12\'.fx.gravity.crosschain.v1.MsgReDelegate\x1a/.fx.gravity.crosschain.v1.MsgReDelegateResponse\x12i\n\x0b\x45\x64itBridger\x12(.fx.gravity.crosschain.v1.MsgEditBridger\x1a\x30.fx.gravity.crosschain.v1.MsgEditBridgerResponse\x12r\n\x0eWithdrawReward\x12+.fx.gravity.crosschain.v1.MsgWithdrawReward\x1a\x33.fx.gravity.crosschain.v1.MsgWithdrawRewardResponse\x12r\n\x0eUnbondedOracle\x12+.fx.gravity.crosschain.v1.MsgUnbondedOracle\x1a\x33.fx.gravity.crosschain.v1.MsgUnbondedOracleResponse\x12x\n\x10OracleSetConfirm\x12-.fx.gravity.crosschain.v1.MsgOracleSetConfirm\x1a\x35.fx.gravity.crosschain.v1.MsgOracleSetConfirmResponse\x12\x86\x01\n\x14OracleSetUpdateClaim\x12\x32.fx.gravity.crosschain.v1.MsgOracleSetUpdatedClaim\x1a:.fx.gravity.crosschain.v1.MsgOracleSetUpdatedClaimResponse\x12x\n\x10\x42ridgeTokenClaim\x12-.fx.gravity.crosschain.v1.MsgBridgeTokenClaim\x1a\x35.fx.gravity.crosschain.v1.MsgBridgeTokenClaimResponse\x12o\n\rSendToFxClaim\x12*.fx.gravity.crosschain.v1.MsgSendToFxClaim\x1a\x32.fx.gravity.crosschain.v1.MsgSendToFxClaimResponse\x12r\n\x0eSendToExternal\x12+.fx.gravity.crosschain.v1.MsgSendToExternal\x1a\x33.fx.gravity.crosschain.v1.MsgSendToExternalResponse\x12\x84\x01\n\x14\x43\x61ncelSendToExternal\x12\x31.fx.gravity.crosschain.v1.MsgCancelSendToExternal\x1a\x39.fx.gravity.crosschain.v1.MsgCancelSendToExternalResponse\x12\x81\x01\n\x13SendToExternalClaim\x12\x30.fx.gravity.crosschain.v1.MsgSendToExternalClaim\x1a\x38.fx.gravity.crosschain.v1.MsgSendToExternalClaimResponse\x12l\n\x0cRequestBatch\x12).fx.gravity.crosschain.v1.MsgRequestBatch\x1a\x31.fx.gravity.crosschain.v1.MsgRequestBatchResponse\x12l\n\x0c\x43onfirmBatch\x12).fx.gravity.crosschain.v1.MsgConfirmBatch\x1a\x31.fx.gravity.crosschain.v1.MsgConfirmBatchResponse\x12l\n\x0cUpdateParams\x12).fx.gravity.crosschain.v1.MsgUpdateParams\x1a\x31.fx.gravity.crosschain.v1.MsgUpdateParamsResponse\x12{\n\x11IncreaseBridgeFee\x12..fx.gravity.crosschain.v1.MsgIncreaseBridgeFee\x1a\x36.fx.gravity.crosschain.v1.MsgIncreaseBridgeFeeResponse\x12~\n\x12UpdateChainOracles\x12/.fx.gravity.crosschain.v1.MsgUpdateChainOracles\x1a\x37.fx.gravity.crosschain.v1.MsgUpdateChainOraclesResponseB1Z/github.com/functionx/fx-core/x/crosschain/typesb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'fx.crosschain.v1.tx_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z/github.com/functionx/fx-core/x/crosschain/types'
  _MSGBONDEDORACLE.fields_by_name['delegate_amount']._options = None
  _MSGBONDEDORACLE.fields_by_name['delegate_amount']._serialized_options = b'\310\336\037\000'
  _MSGADDDELEGATE.fields_by_name['amount']._options = None
  _MSGADDDELEGATE.fields_by_name['amount']._serialized_options = b'\310\336\037\000'
  _MSGORACLESETUPDATEDCLAIM.fields_by_name['members']._options = None
  _MSGORACLESETUPDATEDCLAIM.fields_by_name['members']._serialized_options = b'\310\336\037\000'
  _MSGSENDTOFXCLAIM.fields_by_name['amount']._options = None
  _MSGSENDTOFXCLAIM.fields_by_name['amount']._serialized_options = b'\310\336\037\000\332\336\037&github.com/cosmos/cosmos-sdk/types.Int'
  _MSGSENDTOEXTERNAL.fields_by_name['amount']._options = None
  _MSGSENDTOEXTERNAL.fields_by_name['amount']._serialized_options = b'\310\336\037\000'
  _MSGSENDTOEXTERNAL.fields_by_name['bridge_fee']._options = None
  _MSGSENDTOEXTERNAL.fields_by_name['bridge_fee']._serialized_options = b'\310\336\037\000'
  _MSGREQUESTBATCH.fields_by_name['minimum_fee']._options = None
  _MSGREQUESTBATCH.fields_by_name['minimum_fee']._serialized_options = b'\310\336\037\000\332\336\037&github.com/cosmos/cosmos-sdk/types.Int'
  _MSGREQUESTBATCH.fields_by_name['base_fee']._options = None
  _MSGREQUESTBATCH.fields_by_name['base_fee']._serialized_options = b'\310\336\037\000\332\336\037&github.com/cosmos/cosmos-sdk/types.Int'
  _MSGSETORCHESTRATORADDRESS.fields_by_name['deposit']._options = None
  _MSGSETORCHESTRATORADDRESS.fields_by_name['deposit']._serialized_options = b'\310\336\037\000'
  _MSGADDORACLEDEPOSIT.fields_by_name['amount']._options = None
  _MSGADDORACLEDEPOSIT.fields_by_name['amount']._serialized_options = b'\310\336\037\000'
  _MSGUPDATEPARAMS.fields_by_name['authority']._options = None
  _MSGUPDATEPARAMS.fields_by_name['authority']._serialized_options = b'\322\264-\024cosmos.AddressString'
  _MSGUPDATEPARAMS.fields_by_name['params']._options = None
  _MSGUPDATEPARAMS.fields_by_name['params']._serialized_options = b'\310\336\037\000'
  _MSGUPDATEPARAMS._options = None
  _MSGUPDATEPARAMS._serialized_options = b'\202\347\260*\tauthority'
  _MSGINCREASEBRIDGEFEE.fields_by_name['add_bridge_fee']._options = None
  _MSGINCREASEBRIDGEFEE.fields_by_name['add_bridge_fee']._serialized_options = b'\310\336\037\000'
  _MSGUPDATECHAINORACLES.fields_by_name['authority']._options = None
  _MSGUPDATECHAINORACLES.fields_by_name['authority']._serialized_options = b'\322\264-\024cosmos.AddressString'
  _MSGUPDATECHAINORACLES._options = None
  _MSGUPDATECHAINORACLES._serialized_options = b'\202\347\260*\tauthority'
  _globals['_MSGBONDEDORACLE']._serialized_start=192
  _globals['_MSGBONDEDORACLE']._serialized_end=389
  _globals['_MSGBONDEDORACLERESPONSE']._serialized_start=391
  _globals['_MSGBONDEDORACLERESPONSE']._serialized_end=416
  _globals['_MSGADDDELEGATE']._serialized_start=418
  _globals['_MSGADDDELEGATE']._serialized_end=527
  _globals['_MSGADDDELEGATERESPONSE']._serialized_start=529
  _globals['_MSGADDDELEGATERESPONSE']._serialized_end=553
  _globals['_MSGREDELEGATE']._serialized_start=555
  _globals['_MSGREDELEGATE']._serialized_end=641
  _globals['_MSGREDELEGATERESPONSE']._serialized_start=643
  _globals['_MSGREDELEGATERESPONSE']._serialized_end=666
  _globals['_MSGEDITBRIDGER']._serialized_start=668
  _globals['_MSGEDITBRIDGER']._serialized_end=753
  _globals['_MSGEDITBRIDGERRESPONSE']._serialized_start=755
  _globals['_MSGEDITBRIDGERRESPONSE']._serialized_end=779
  _globals['_MSGUNBONDEDORACLE']._serialized_start=781
  _globals['_MSGUNBONDEDORACLE']._serialized_end=844
  _globals['_MSGUNBONDEDORACLERESPONSE']._serialized_start=846
  _globals['_MSGUNBONDEDORACLERESPONSE']._serialized_end=873
  _globals['_MSGWITHDRAWREWARD']._serialized_start=875
  _globals['_MSGWITHDRAWREWARD']._serialized_end=938
  _globals['_MSGWITHDRAWREWARDRESPONSE']._serialized_start=940
  _globals['_MSGWITHDRAWREWARDRESPONSE']._serialized_end=967
  _globals['_MSGORACLESETCONFIRM']._serialized_start=969
  _globals['_MSGORACLESETCONFIRM']._serialized_end=1095
  _globals['_MSGORACLESETCONFIRMRESPONSE']._serialized_start=1097
  _globals['_MSGORACLESETCONFIRMRESPONSE']._serialized_end=1126
  _globals['_MSGORACLESETUPDATEDCLAIM']._serialized_start=1129
  _globals['_MSGORACLESETUPDATEDCLAIM']._serialized_end=1335
  _globals['_MSGORACLESETUPDATEDCLAIMRESPONSE']._serialized_start=1337
  _globals['_MSGORACLESETUPDATEDCLAIMRESPONSE']._serialized_end=1371
  _globals['_MSGSENDTOFXCLAIM']._serialized_start=1374
  _globals['_MSGSENDTOFXCLAIM']._serialized_end=1622
  _globals['_MSGSENDTOFXCLAIMRESPONSE']._serialized_start=1624
  _globals['_MSGSENDTOFXCLAIMRESPONSE']._serialized_end=1650
  _globals['_MSGSENDTOEXTERNAL']._serialized_start=1653
  _globals['_MSGSENDTOEXTERNAL']._serialized_end=1824
  _globals['_MSGSENDTOEXTERNALRESPONSE']._serialized_start=1826
  _globals['_MSGSENDTOEXTERNALRESPONSE']._serialized_end=1877
  _globals['_MSGCANCELSENDTOEXTERNAL']._serialized_start=1879
  _globals['_MSGCANCELSENDTOEXTERNAL']._serialized_end=1964
  _globals['_MSGCANCELSENDTOEXTERNALRESPONSE']._serialized_start=1966
  _globals['_MSGCANCELSENDTOEXTERNALRESPONSE']._serialized_end=1999
  _globals['_MSGREQUESTBATCH']._serialized_start=2002
  _globals['_MSGREQUESTBATCH']._serialized_end=2226
  _globals['_MSGREQUESTBATCHRESPONSE']._serialized_start=2228
  _globals['_MSGREQUESTBATCHRESPONSE']._serialized_end=2274
  _globals['_MSGCONFIRMBATCH']._serialized_start=2277
  _globals['_MSGCONFIRMBATCH']._serialized_end=2423
  _globals['_MSGCONFIRMBATCHRESPONSE']._serialized_start=2425
  _globals['_MSGCONFIRMBATCHRESPONSE']._serialized_end=2450
  _globals['_MSGSENDTOEXTERNALCLAIM']._serialized_start=2453
  _globals['_MSGSENDTOEXTERNALCLAIM']._serialized_end=2610
  _globals['_MSGSENDTOEXTERNALCLAIMRESPONSE']._serialized_start=2612
  _globals['_MSGSENDTOEXTERNALCLAIMRESPONSE']._serialized_end=2644
  _globals['_MSGBRIDGETOKENCLAIM']._serialized_start=2647
  _globals['_MSGBRIDGETOKENCLAIM']._serialized_end=2849
  _globals['_MSGBRIDGETOKENCLAIMRESPONSE']._serialized_start=2851
  _globals['_MSGBRIDGETOKENCLAIMRESPONSE']._serialized_end=2880
  _globals['_MSGSETORCHESTRATORADDRESS']._serialized_start=2883
  _globals['_MSGSETORCHESTRATORADDRESS']._serialized_end=3055
  _globals['_MSGADDORACLEDEPOSIT']._serialized_start=3057
  _globals['_MSGADDORACLEDEPOSIT']._serialized_end=3171
  _globals['_MSGUPDATEPARAMS']._serialized_start=3174
  _globals['_MSGUPDATEPARAMS']._serialized_end=3328
  _globals['_MSGUPDATEPARAMSRESPONSE']._serialized_start=3330
  _globals['_MSGUPDATEPARAMSRESPONSE']._serialized_end=3355
  _globals['_MSGINCREASEBRIDGEFEE']._serialized_start=3358
  _globals['_MSGINCREASEBRIDGEFEE']._serialized_end=3497
  _globals['_MSGINCREASEBRIDGEFEERESPONSE']._serialized_start=3499
  _globals['_MSGINCREASEBRIDGEFEERESPONSE']._serialized_end=3529
  _globals['_MSGUPDATECHAINORACLES']._serialized_start=3531
  _globals['_MSGUPDATECHAINORACLES']._serialized_end=3652
  _globals['_MSGUPDATECHAINORACLESRESPONSE']._serialized_start=3654
  _globals['_MSGUPDATECHAINORACLESRESPONSE']._serialized_end=3685
  _globals['_MSG']._serialized_start=3688
  _globals['_MSG']._serialized_end=5813
# @@protoc_insertion_point(module_scope)



from decimal import Decimal
from typing import Optional

from fxsdk.client.grpc import Client, GRPCBlockHeightHeader
from fxsdk.dec import dec_from_str
from fxsdk.msg.marginx import new_position_from_proto, new_order_from_proto, Order, Position, \
    new_pair_funding_rate_from_proto, PairFundingRate, OrderDepths, PairPrice, new_order_depths_from_proto, DecCoin

from fxsdk.x.marginx.oracle.v1.query_pb2 import QueryPriceRequest, QueryMarketRequest
from fxsdk.x.marginx.oracle.v1.types_pb2 import Market
from fxsdk.x.marginx.oracle.v1.query_pb2_grpc import QueryStub as OracleClient

from fxsdk.x.fx.dex.v1.funding_pb2 import Funding
from fxsdk.x.fx.dex.v1.query_pb2_grpc import QueryStub as DexClient
from fxsdk.x.fx.dex.v1.query_pb2 import QueryPositionReq, QueryOrderRequest, QueryOrdersRequest, QueryFundingReq, \
    QueryPairFundingRatesReq, QueryOrderbookReq, QueryMarkPriceReq


class MarginXClient(Client):

    def __init__(self, url: str = 'localhost:9090'):
        super().__init__(url)

    def query_dec_balance(self, address: str, denom: str, height: Optional[int] = 0) -> DecCoin:
        balance = self.query_balance(address, denom, height)
        return DecCoin(balance.denom, dec_from_str(balance.amount))

    def query_all_dec_balances(self, address: str, height: Optional[int] = 0) -> [DecCoin]:
        balances = self.query_all_balances(address, height)
        coins = []
        for balance in balances:
            coins.append(DecCoin(balance.denom, dec_from_str(balance.amount)))
        return coins

    def query_oracle_price(self, pair_id: str, height: Optional[int] = 0) -> Decimal:
        metadata = [(GRPCBlockHeightHeader, str(height))]
        response = OracleClient(self.channel).Price(QueryPriceRequest(pair_id=pair_id), metadata=metadata)
        return dec_from_str(response.price)

    def query_oracle_prices(self, height: Optional[int] = 0) -> [PairPrice]:
        metadata = [(GRPCBlockHeightHeader, str(height))]
        response = OracleClient(self.channel).Prices(QueryPriceRequest(), metadata=metadata)
        pair_prices = []
        for price in response.prices:
            pair_prices.append(PairPrice(price.pair_id, dec_from_str(price.price)))
        return pair_prices

    def query_oracle_market(self, pair_id: str, height: Optional[int] = 0) -> Market:
        metadata = [(GRPCBlockHeightHeader, str(height))]
        response = OracleClient(self.channel).Market(QueryMarketRequest(pair_id=pair_id), metadata=metadata)
        return response.market

    def query_oracle_markets(self, height: Optional[int] = 0) -> [Market]:
        metadata = [(GRPCBlockHeightHeader, str(height))]
        response = OracleClient(self.channel).Markets(QueryMarketRequest(), metadata=metadata)
        return response.markets

    def query_positions(self, owner: str, pair_id: str, height: Optional[int] = 0) -> [Position]:
        metadata = [(GRPCBlockHeightHeader, str(height))]
        response = DexClient(self.channel).QueryPosition(QueryPositionReq(owner=owner, pair_id=pair_id),
                                                         metadata=metadata)
        positions = []
        for position in response.positions:
            positions.append(new_position_from_proto(position))
        return positions

    def query_order(self, order_id: str, height: Optional[int] = 0) -> Order:
        metadata = [(GRPCBlockHeightHeader, str(height))]
        response = DexClient(self.channel).QueryOrder(QueryOrderRequest(order_id=order_id), metadata=metadata)
        order = new_order_from_proto(response.order)
        return order

    def query_orders(self, owner: str, pair_id: str, height: Optional[int] = 0) -> [Order]:
        metadata = [(GRPCBlockHeightHeader, str(height))]
        response = DexClient(self.channel).QueryOrders(QueryOrdersRequest(owner=owner, pair_id=pair_id),
                                                       metadata=metadata)
        orders = []
        for order in response.orders:
            orders.append(new_order_from_proto(order))
        return orders

    def query_funding_info(self, height: Optional[int] = 0) -> Funding:
        metadata = [(GRPCBlockHeightHeader, str(height))]
        response = DexClient(self.channel).QueryFunding(QueryFundingReq(), metadata=metadata)
        return response.funding

    def query_funding_rates(self, last_or_realtime: bool = True, height: Optional[int] = 0) -> [PairFundingRate]:
        metadata = [(GRPCBlockHeightHeader, str(height))]
        response = DexClient(self.channel).QueryPairFundingRates(
            QueryPairFundingRatesReq(last_or_realtime=last_or_realtime), metadata=metadata)
        pair_funding_rates = []
        for rate in response.pair_funding_rates:
            pair_funding_rates.append(new_pair_funding_rate_from_proto(rate))
        return pair_funding_rates

    def query_order_depths(self, pair_id: str, height: Optional[int] = 0) -> OrderDepths:
        metadata = [(GRPCBlockHeightHeader, str(height))]
        response = DexClient(self.channel).QueryOrderbook(QueryOrderbookReq(pair_id=pair_id), metadata=metadata)
        return new_order_depths_from_proto(response.Asks, response.Bids)

    def query_mark_price(self, paid_id: str, height: Optional[int] = 0) -> Decimal:
        metadata = [(GRPCBlockHeightHeader, str(height))]
        response = DexClient(self.channel).QueryMarkPrice(QueryMarkPriceReq(pair_id=paid_id), metadata=metadata)
        if len(response.pair_mark_price) > 0:
            return dec_from_str(response.pair_mark_price[0].price)
        else:
            raise Exception("No mark price found for pair_id: {}".format(paid_id))

    def query_mark_prices(self, height: Optional[int] = 0) -> [PairPrice]:
        metadata = [(GRPCBlockHeightHeader, str(height))]
        response = DexClient(self.channel).QueryMarkPrice(QueryMarkPriceReq(query_all=True), metadata=metadata)
        pair_prices = []
        for price in response.pair_mark_price:
            pair_prices.append(PairPrice(price.pair_id, dec_from_str(price.price)))
        return pair_prices



#!/usr/bin/env python
from setuptools import (
    find_packages,
    setup,
)

requires = [
    "grpcio>=1.59.0",
    "google-api-python-client>=2.102.0",
    "bech32>=1.2.0",
    "ecdsa>=0.18.0",
    "hdwallets>=0.1.2",
    "mnemonic>=0.20",
    "eth-utils>=2.2.2",
    "coincurve>=18.0.0",
    "pycryptodome>=3.19.0"
]

with open("README.md", "r", encoding="UTF-8") as fh:
    long_description = fh.read()

setup(
    name="fx-py-sdk",
    version="0.7.3",
    platforms='any',
    description="Python library for FunctionX",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='zakir',
    packages=find_packages(),
    python_requires=">=3.7",
    install_requires=requires,
    url='https://github.com/functionx/fx-py-sdk',
    keywords=["FunctionX", "blockchain"],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.11",
    ],
    include_package_data=True
)
